@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -346,14 +346,15 @@ select_basis_set(int atomIndex,
     @return the number of basis set shells.
 */
 static int 
-setup_shells_multi_basis_getcount(const Molecule& molecule,
+setup_shells_multi_basis_getcount(const Atom* atomList,
+				  int noOfAtoms,
 				  const basisset_struct* basissetDefault,
 				  int noOfRanges,
 				  const basis_set_range_struct* rangeList)
 {
   int noOfShells = 0;
-  for(int i = 0; i < molecule.noOfAtoms; i++) {
-    int z = (int)molecule.atoms[i].charge;
+  for(int i = 0; i < noOfAtoms; i++) {
+    int z = (int)atomList[i].charge;
     const basisset_struct* basissetCurrAtom = select_basis_set(i, noOfRanges, rangeList, basissetDefault);
     int noOfShellsCurrAtom = basissetCurrAtom->atoms[z].noOfShells;
     if(noOfShellsCurrAtom <= 0) {
@@ -370,7 +371,8 @@ setup_shells_multi_basis_getcount(const Molecule& molecule,
 
 static int
 setup_shells_multi_basis(const IntegralInfo& integralInfo,
-			 const Molecule& molecule, 
+			 const Atom* atomList,
+			 int noOfAtoms,
 			 const basisset_struct* basissetDefault, 
 			 ShellSpecStruct* shell_list,
 			 int noOfShells,
@@ -382,8 +384,8 @@ setup_shells_multi_basis(const IntegralInfo& integralInfo,
   
   int count = 0;
   SquareFuncIntegrator sfi;
-  for(int i = 0; i < molecule.noOfAtoms; i++) {
-    int z = (int)molecule.atoms[i].charge;
+  for(int i = 0; i < noOfAtoms; i++) {
+    int z = (int)atomList[i].charge;
     const basisset_struct* basissetCurrAtom = select_basis_set(i, noOfRanges, rangeList, basissetDefault);
     int noOfShellsCurrAtom = basissetCurrAtom->atoms[z].noOfShells;
     if(noOfShellsCurrAtom <= 0) {
@@ -411,7 +413,7 @@ setup_shells_multi_basis(const IntegralInfo& integralInfo,
       shell_list[count].shell_ID  = basissetShell->shell_ID;
       for(int k = 0; k < 3; k++)
 	shell_list[count].centerCoords[k] = 
-	  molecule.atoms[i].coords[k];
+	  atomList[i].coords[k];
       for(int k = 0; k < basissetShell->contrCount; k++) {
 	ergo_real exponent = basissetShell->exponentList[k];
 	shell_list[count].coeffList[k] = 
@@ -435,6 +437,77 @@ setup_shells_multi_basis(const IntegralInfo& integralInfo,
 
 
 
+void BasisInfoStruct::addBasisfuncsForAtomList(const Atom* atomList,
+					       int noOfAtoms,
+					       const basisset_struct* basissetDefault,
+					       int noOfRanges,
+					       const basis_set_range_struct* rangeList,
+					       const IntegralInfo& integralInfo,
+					       int print_raw,
+					       int do_normalization,
+					       int skip_sort_shells) {
+
+  int noOfShellsToAdd = setup_shells_multi_basis_getcount(atomList,
+							  noOfAtoms,
+                                                          basissetDefault,
+                                                          noOfRanges,
+                                                          rangeList);
+  if(noOfShellsToAdd <= 0)
+    throw std::runtime_error("error in setup_shells_multi_basis_getcount");
+
+  int noOfShellsNew = noOfShells + noOfShellsToAdd;
+  do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, 
+	    "BasisInfoStruct::addBasisfuncsForAtomList, noOfShellsNew = %i", noOfShellsNew);
+  
+  ShellSpecStruct* shell_list_new = new ShellSpecStruct[noOfShellsNew];
+  if(this->noOfShells > 0)
+    memcpy(shell_list_new, this->shellList, this->noOfShells*sizeof(ShellSpecStruct));
+  
+  // Setup new shells
+  if(setup_shells_multi_basis(integralInfo,
+			      atomList,
+			      noOfAtoms,
+                              basissetDefault,
+                              &shell_list_new[noOfShells],
+                              noOfShellsToAdd,
+                              noOfRanges,
+                              rangeList,
+                              use_6_d_funcs) != 0)
+    throw std::runtime_error("error in setup_shells_multi_basis_getcount");
+
+  for(int i = 0; i < noOfShellsNew; i++) {
+    for(int kk = 0; kk < shell_list_new[i].noOfContr; kk++) {
+      // calculate size
+      shell_list_new[i].sizeList[kk] = 
+	std::fabs(std::pow((ergo_real)pi/shell_list_new[i].exponentList[kk], (ergo_real)1.5) * shell_list_new[i].coeffList[kk]);
+    }
+  } /* END FOR i */
+  
+  if(skip_sort_shells == 0) {
+    /* sort shells by shell ID */
+    std::vector<ShellSpecStruct> shellListTemp(noOfShellsNew);
+    if(sort_shells(shell_list_new, &shellListTemp[0], noOfShellsNew) != 0)
+      throw std::runtime_error("Error in sort_shells.");
+  }
+
+  if(this->shellList)
+    delete this->shellList;
+  this->shellList = shell_list_new;
+  this->noOfShells = noOfShellsNew;
+  
+  if(do_normalization) {
+    if(this->normalizeShells(integralInfo) != 0)
+      throw std::runtime_error("error in normalizeShells");
+  }
+  if(this->get_basis_funcs() != 0)
+    throw std::runtime_error("error in get_basis_funcs");
+  do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "get_basis_funcs returned OK, number of basis funcs: %i",
+	    this->noOfBasisFuncs);
+  if(this->getSimplePrimitivesAll(integralInfo) != 0)
+    throw std::runtime_error("error in getSimplePrimitivesAll");
+  do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "getSimplePrimitivesAll returned OK, n = %i",
+	    this->noOfSimplePrimitives);
+}
 
 
 
@@ -486,7 +559,7 @@ int BasisInfoStruct::addBasisfuncsForMolecule(const Molecule& molecule,
     ERGO_SPREFIX
   };
   basisset_struct* basissetDefault = new basisset_struct;
-  memset(basissetDefault, 0, sizeof(basissetDefault));
+  memset(basissetDefault, 0, sizeof(basisset_struct));
   
   basis_set_range_struct rangeListTemp[noOfRanges];
   memset(rangeListTemp, 0, noOfRanges*sizeof(basis_set_range_struct));
@@ -522,80 +595,17 @@ int BasisInfoStruct::addBasisfuncsForMolecule(const Molecule& molecule,
     }
   }
 
-  int noOfShellsToAdd = setup_shells_multi_basis_getcount(molecule,
-                                                          basissetDefault,
-                                                          noOfRanges,
-                                                          rangeListTemp);
-  if(noOfShellsToAdd <= 0) {
-    do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in setup_shells_multi_basis_getcount");
-    delete basissetDefault;
-    return -1;
-  }
+  addBasisfuncsForAtomList(molecule.getAtomListPtr(),
+			   molecule.getNoOfAtoms(),
+			   basissetDefault,
+			   noOfRanges,
+			   rangeListTemp,
+			   integralInfo,
+			   print_raw,
+			   do_normalization,
+			   skip_sort_shells);
 
-  int noOfShellsNew = noOfShells + noOfShellsToAdd;
-
-  do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, 
-	    "BasisInfoStruct::addBasisfuncsForMolecule, noOfShellsNew = %i", noOfShellsNew);
-  
-  ShellSpecStruct* shell_list_new = new ShellSpecStruct[noOfShellsNew];
-  if(this->noOfShells > 0)
-    memcpy(shell_list_new, this->shellList, this->noOfShells*sizeof(ShellSpecStruct));
-  
-  // Setup new shells
-  if(setup_shells_multi_basis(integralInfo,
-			      molecule,
-                              basissetDefault,
-                              &shell_list_new[noOfShells],
-                              noOfShellsToAdd,
-                              noOfRanges,
-                              rangeListTemp,
-                              use_6_d_funcs) != 0)
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in setup_shells_multi_basis");
-      delete basissetDefault;
-      return -1;
-    }
   delete basissetDefault;
-
-  for(int i = 0; i < noOfShellsNew; i++) {
-    for(int kk = 0; kk < shell_list_new[i].noOfContr; kk++) {
-      // calculate size
-      shell_list_new[i].sizeList[kk] = 
-	std::fabs(std::pow((ergo_real)pi/shell_list_new[i].exponentList[kk], (ergo_real)1.5) * shell_list_new[i].coeffList[kk]);
-    }
-  } /* END FOR i */
-  
-  if(skip_sort_shells == 0) {
-    /* sort shells by shell ID */
-    ShellSpecStruct* shellListTemp = new ShellSpecStruct [noOfShellsNew];
-    if(sort_shells(shell_list_new, shellListTemp, noOfShellsNew) != 0)
-      return -1;
-    delete []shellListTemp;
-  }
-
-  if(this->shellList)
-    delete this->shellList;
-  this->shellList = shell_list_new;
-  this->noOfShells = noOfShellsNew;
-  
-  if(do_normalization) {
-    if(this->normalizeShells(integralInfo) != 0) {
-      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in normalizeShells");
-      return -1;
-    }
-  }
-  if(this->get_basis_funcs() != 0) {
-    do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in get_basis_funcs");
-    return -1;
-  }
-  do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "get_basis_funcs returned OK, number of basis funcs: %i",
-	    this->noOfBasisFuncs);
-  if(this->getSimplePrimitivesAll(integralInfo) != 0) {
-    do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in getSimplePrimitivesAll");
-    return -1;
-  }
-  do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "getSimplePrimitivesAll returned OK, n = %i",
-	    this->noOfSimplePrimitives);
   
   return 0;
 }
@@ -796,6 +806,108 @@ int BasisInfoStruct::getSimplePrimitivesAll(const IntegralInfo& integralInfo)
   return 0;
 }
 
+/** Initializes all the fields to sane values. */
+BasisInfoStruct::BasisInfoStruct(int use_6_d_funcs_) : 
+  use_6_d_funcs(use_6_d_funcs_),
+  noOfShells(0),
+  shellList(NULL),
+  noOfBasisFuncs(0),
+  basisFuncList(NULL),
+  noOfSimplePrimitives(0),
+  simplePrimitiveList(NULL)
+{
+}
+
+/** Copies values from another BasisInfoStruct. */
+BasisInfoStruct::BasisInfoStruct(const BasisInfoStruct & b) : 
+  use_6_d_funcs(b.use_6_d_funcs),
+  noOfShells(b.noOfShells),
+  noOfBasisFuncs(b.noOfBasisFuncs),
+  noOfSimplePrimitives(b.noOfSimplePrimitives)
+{
+  shellList = new ShellSpecStruct[noOfShells];
+  memcpy(shellList, b.shellList, noOfShells*sizeof(ShellSpecStruct));
+  basisFuncList = new BasisFuncStruct[noOfBasisFuncs];
+  memcpy(basisFuncList, b.basisFuncList, noOfBasisFuncs*sizeof(BasisFuncStruct));
+  simplePrimitiveList = new DistributionSpecStruct[noOfSimplePrimitives];
+  memcpy(simplePrimitiveList, b.simplePrimitiveList, noOfSimplePrimitives*sizeof(DistributionSpecStruct));
+}
+
+
+/** Function needed for Chunks&Tasks usage. */
+void BasisInfoStruct::write_to_buffer ( char * dataBuffer, size_t const bufferSize ) const {
+  // First store the 4 int numbers.
+  char* p = dataBuffer;
+  if(bufferSize < get_size())
+    throw std::runtime_error("Error: bufferSize too small.");
+  memcpy(p, &noOfShells, sizeof(int));
+  p += sizeof(int);
+  memcpy(p, &noOfBasisFuncs, sizeof(int));
+  p += sizeof(int);
+  memcpy(p, &noOfSimplePrimitives, sizeof(int));
+  p += sizeof(int);
+  memcpy(p, &use_6_d_funcs, sizeof(int));
+  p += sizeof(int);  
+  // There are three lists that need to be stored. Take care of them one by one.
+  // shellList
+  memcpy(p, shellList, noOfShells * sizeof(ShellSpecStruct));
+  p += noOfShells * sizeof(ShellSpecStruct);
+  // basisFuncList
+  memcpy(p, basisFuncList, noOfBasisFuncs * sizeof(BasisFuncStruct));
+  p += noOfBasisFuncs * sizeof(BasisFuncStruct);
+  // simplePrimitiveList
+  memcpy(p, simplePrimitiveList, noOfSimplePrimitives * sizeof(DistributionSpecStruct));
+  p += noOfSimplePrimitives * sizeof(DistributionSpecStruct);
+  // DONE!
+}
+
+/** Function needed for Chunks&Tasks usage. */
+size_t BasisInfoStruct::get_size() const {
+  return 4 * sizeof(int) 
+    + noOfShells * sizeof(ShellSpecStruct)
+    + noOfBasisFuncs * sizeof(BasisFuncStruct)
+    + noOfSimplePrimitives * sizeof(DistributionSpecStruct);
+}
+
+/** Function needed for Chunks&Tasks usage. */
+void BasisInfoStruct::assign_from_buffer ( char const * dataBuffer, size_t const bufferSize) {
+  // First get the 4 int numbers.
+  const char* p = dataBuffer;
+  if(bufferSize < 4 * sizeof(int))
+    throw std::runtime_error("Error: bufferSize too small.");
+  memcpy(&noOfShells, p, sizeof(int));
+  p += sizeof(int);
+  memcpy(&noOfBasisFuncs, p, sizeof(int));
+  p += sizeof(int);
+  memcpy(&noOfSimplePrimitives, p, sizeof(int));
+  p += sizeof(int);
+  memcpy(&use_6_d_funcs, p, sizeof(int));
+  p += sizeof(int);  
+  // There are three lists that need to be set up. Take care of them one by one.
+  // shellList
+  shellList = new ShellSpecStruct[noOfShells];
+  memcpy(shellList, p, noOfShells * sizeof(ShellSpecStruct));
+  p += noOfShells * sizeof(ShellSpecStruct);
+  // basisFuncList
+  basisFuncList = new BasisFuncStruct[noOfBasisFuncs];
+  memcpy(basisFuncList, p, noOfBasisFuncs * sizeof(BasisFuncStruct));
+  p += noOfBasisFuncs * sizeof(BasisFuncStruct);
+  // simplePrimitiveList
+  simplePrimitiveList = new DistributionSpecStruct[noOfSimplePrimitives];
+  memcpy(simplePrimitiveList, p, noOfSimplePrimitives * sizeof(DistributionSpecStruct));
+  p += noOfSimplePrimitives * sizeof(DistributionSpecStruct);
+  // DONE!
+  if(static_cast<size_t>(p-dataBuffer) > bufferSize)
+    throw std::runtime_error("Error: (p > bufferSize).");
+}
+
+BasisInfoStruct::~BasisInfoStruct()
+{
+  if(shellList)                 delete [] shellList;
+  if(basisFuncList)             delete [] basisFuncList;
+  if(simplePrimitiveList)       delete [] simplePrimitiveList;
+}
+
 
 /** Compute safe upper limit for largest possible distance between any
     two basis functions in given basis set.
