@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -122,23 +122,22 @@ struct BasisInfoStruct{
   DistributionSpecStruct* simplePrimitiveList;
 
   /** Initializes all the fields to sane values. */
-  BasisInfoStruct(int use_6_d_funcs_ = 0) : 
-       use_6_d_funcs(use_6_d_funcs_),
-       noOfShells(0),
-       shellList(NULL),
-       noOfBasisFuncs(0),
-       basisFuncList(NULL),
-       noOfSimplePrimitives(0),
-       simplePrimitiveList(NULL)
-  {
-  }
+  BasisInfoStruct(int use_6_d_funcs_ = 0);
 
-  ~BasisInfoStruct()
-  {
-    if(shellList)                 delete [] shellList;
-    if(basisFuncList)             delete [] basisFuncList;
-    if(simplePrimitiveList)       delete [] simplePrimitiveList;
-  }
+  /** Copies values from another BasisInfoStruct. */
+  BasisInfoStruct(const BasisInfoStruct & b);
+
+  ~BasisInfoStruct();
+
+  void addBasisfuncsForAtomList(const Atom* atomList,
+				int noOfAtoms,
+				const basisset_struct* basissetDefault,
+				int noOfRanges,
+				const basis_set_range_struct* rangeList,
+				const IntegralInfo & integralInfo,
+				int print_raw,
+				int do_normalization,
+				int skip_sort_shells);
 
   int addBasisfuncsForMolecule(const Molecule& molecule,
 				  const char* basisset_filename_default,
@@ -157,6 +156,11 @@ struct BasisInfoStruct{
   int get_basis_funcs();
 
   int getSimplePrimitivesAll(const IntegralInfo& integralInfo);
+
+  // Stuff needed for Chunks&Tasks usage
+  void write_to_buffer ( char * dataBuffer, size_t const bufferSize ) const;
+  size_t get_size() const;
+  void assign_from_buffer ( char const * dataBuffer, size_t const bufferSize);
 };
 
 
