@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,9 +43,14 @@
 #include "TC2.h"
 #include "PuriInfo.h"
 #include "Purification.h"
+#include "Purification_scaled.h"
 #include "units.h"
 #include "machine_epsilon.h"
 #include "simple_purification.h"
+#include "puri_mmul_tests.h"
+#include "AllocatorManager.h"
+
+#define RUN_NEW_PURIFICATION 0
 
 typedef mat::DebugLevelLow debugPolicy;
 
@@ -170,17 +175,25 @@ int get_dens_from_fock_sparse(int n,
 			      int do_sparsity_investigation,
 			      int sparsity_plots_resolution_m,
 			      int do_comparison_to_simple_purification,
+			      int do_puri_mmul_tests,
 			      generalVector * eigVecLUMO,
 			      generalVector * eigVecHOMO)
 {
   Util::TimeMeter timeMeterTot;
 
   do_output(LOG_CAT_INFO, LOG_AREA_DENSFROMF, "get_dens_from_fock_sparse() start!");
+
+  std::string allocStatsStr1 = mat::AllocatorManager<ergo_real>::instance().getStatistics();
+  do_output(LOG_CAT_INFO, LOG_AREA_DENSFROMF, "Before writeAndReadAll(): %s", allocStatsStr1.c_str());
+
   Util::TimeMeter timeMeterWriteAndReadAll;
   std::string sizesStr = mat::FileWritable::writeAndReadAll();
   timeMeterWriteAndReadAll.print(LOG_AREA_DENSFROMF, "FileWritable::writeAndReadAll");
   do_output(LOG_CAT_INFO, LOG_AREA_DENSFROMF, ((std::string)"writeAndReadAll sizesStr: '" + sizesStr).c_str());
   
+  std::string allocStatsStr2 = mat::AllocatorManager<ergo_real>::instance().getStatistics();
+  do_output(LOG_CAT_INFO, LOG_AREA_DENSFROMF, "After writeAndReadAll(): %s", allocStatsStr2.c_str());
+
   if(noOfOccupiedOrbs == 0)
     {
       do_output(LOG_CAT_INFO, LOG_AREA_DENSFROMF, "get_dens_from_fock_sparse: (noOfOccupiedOrbs == 0), skipping.");
@@ -202,6 +215,8 @@ int get_dens_from_fock_sparse(int n,
  
   symmMatrix F(Finput);
   F.readFromFile();
+  std::string allocStatsStr3 = mat::AllocatorManager<ergo_real>::instance().getStatistics();
+  do_output(LOG_CAT_INFO, LOG_AREA_DENSFROMF, "After F.readFromFile(): %s", allocStatsStr3.c_str());
  
   {
     triangMatrix invCholFactor_tmp(invCholFactor);
@@ -302,6 +317,13 @@ int get_dens_from_fock_sparse(int n,
   // eigenvalues of F_ort_prev but improved values will hopefully be
   // calculated in purification.
   
+#if RUN_NEW_PURIFICATION
+  /*******    CODE FOR NEW PURIFICATION     *******/
+  intervalType homoIntervalSaved = homoInterval_F_ort_prev;
+  intervalType lumoIntervalSaved = lumoInterval_F_ort_prev;
+  /******* END OF CODE FOR NEW PURIFICATION *******/
+#endif  
+
   
   if(noOfOccupiedOrbs == n)
     {
@@ -579,6 +601,24 @@ int get_dens_from_fock_sparse(int n,
       puri_stats[stats_prefix + "HOMO_LUMO_gap_hi_eV"]   = gapMax / UNIT_one_eV;
 
       F.transfer(resultDens);
+
+      if ( do_puri_mmul_tests == 1 ) { // Code to do some matrix-matrix multiplication tests
+	// Copy F
+	symmMatrix F_for_mmul_tests(F_ort_prev);
+	F_for_mmul_tests.readFromFile();
+	// Set up poly_choices vector
+	std::vector<int> poly_choices;
+	puriInfo.getPolys(poly_choices);
+	// Set up thresh_values vector
+	std::vector<ergo_real> thresh_values;
+	puriInfo.getThreshValues(thresh_values);
+	run_puri_mmul_tests(F_for_mmul_tests, 
+			    allEigsInterval,
+			    poly_choices,
+			    thresh_values,
+			    resultDens,
+			    globalCounter);
+      }
     }
 
   if ( do_comparison_to_simple_purification == 1 ) { // Code to test simple purification
@@ -592,6 +632,78 @@ int get_dens_from_fock_sparse(int n,
 					   globalCounter );    
   }
 
+#if RUN_NEW_PURIFICATION
+  /*******    CODE FOR NEW PURIFICATION     *******/
+  if ( homoIntervalSaved.upp() < lumoIntervalSaved.low() ) {
+    do_output(LOG_CAT_INFO, LOG_AREA_DENSFROMF, 
+	      "**********************************");
+    do_output(LOG_CAT_INFO, LOG_AREA_DENSFROMF, 
+	      "Running new purification");
+    for (int use_scaling = 0; use_scaling <=1; use_scaling++) {
+      // Run new purification
+      do_output(LOG_CAT_INFO, LOG_AREA_DENSFROMF, 
+		"************** RUN WITH use_scaling = %i.", use_scaling);    
+      // Copy F
+      symmMatrix F_and_D(F_ort_prev);
+      F_and_D.readFromFile();
+      Util::TimeMeter timeMeterNewPuri;
+      pur::Purification_scaled<symmMatrix> myPuri( F_and_D,  
+						   allEigsInterval,
+						   homoIntervalSaved, 
+						   lumoIntervalSaved,
+						   eigvalueThr_Puri,
+						   subspaceThr_Puri,
+						   maxMul,
+						   truncationNormPurification,
+						   use_scaling );
+      myPuri.purify();
+      timeMeterNewPuri.print(LOG_AREA_DENSFROMF, "Puri: constr. + purify()");
+      ergo_real densDiff = symmMatrix::eucl_diff(F_and_D, resultDens, 1e-10);
+      do_output(LOG_CAT_INFO, LOG_AREA_DENSFROMF, 
+		"Density difference: %22.11f.", (double)densDiff);    
+      {
+	std::stringstream ss;
+	ss << "Original HOMO interval: " << homoIntervalSaved;
+	do_output(LOG_CAT_INFO, LOG_AREA_DENSFROMF, ss.str().c_str());
+      }
+      {
+	std::stringstream ss;
+	ss << "Original LUMO interval: " << lumoIntervalSaved;
+	do_output(LOG_CAT_INFO, LOG_AREA_DENSFROMF, ss.str().c_str());
+      }
+      {
+	mat::Interval<ergo_real> hoF_new;
+	mat::Interval<ergo_real> luF_new;
+	myPuri.get_homo_lumo_intervals(hoF_new, luF_new);
+	std::stringstream ss;
+	ss << "Computed HOMO interval: " << hoF_new;
+	do_output(LOG_CAT_INFO, LOG_AREA_DENSFROMF, ss.str().c_str());
+	std::stringstream ss2;
+	ss2 << "Computed LUMO interval: " << luF_new;
+	do_output(LOG_CAT_INFO, LOG_AREA_DENSFROMF, ss2.str().c_str());
+      }
+      {
+	std::stringstream ss;
+	ss << "puriNewInfo_" << globalCounter << "_scal_" << use_scaling << ".m";
+	std::ofstream ff( ss.str().c_str() );
+	myPuri.mInfo(ff);
+	ff.close();
+      }
+      {
+	std::stringstream ss;
+	ss << "puriNewTime_" << globalCounter << "_scal_" << use_scaling << ".m";
+	std::ofstream ff( ss.str().c_str() );
+	myPuri.mTime(ff);
+	ff.close();
+      }
+    }
+    do_output(LOG_CAT_INFO, LOG_AREA_DENSFROMF, 
+	      "**********************************");
+  } // end running new purification
+  /******* END OF CODE FOR NEW PURIFICATION *******/
+#endif
+  
+
   if ( do_sparsity_investigation == 1 ) {
     output_magnitude_histogram( resultDens, stats_prefix + "mag_histogram_D_ort", sparsity_plots_resolution_m);
   }
