@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -222,7 +222,7 @@ dft_get_num_threads(void)
             }
         }
         if(dft_thread_count<=0) dft_thread_count = 1;
-        fort_print("Default number of threads: %d", dft_thread_count);
+        fort_print("Default number of threads: %d\n", dft_thread_count);
     }
     return dft_thread_count;
 }
@@ -310,10 +310,10 @@ ErgoMolInfo::getAtom(int icent, int *cnt, real (*coor)[3],
                      int *charge, int *mult) const
 {
     *cnt = 1; /* one atom in the batch */
-    coor[0][0] = molecule.atoms[icent].coords[0];
-    coor[0][1] = molecule.atoms[icent].coords[1];
-    coor[0][2] = molecule.atoms[icent].coords[2];
-    *charge    = int( molecule.atoms[icent].charge );
+    coor[0][0] = molecule.getAtom(icent).coords[0];
+    coor[0][1] = molecule.getAtom(icent).coords[1];
+    coor[0][2] = molecule.getAtom(icent).coords[2];
+    *charge    = int( molecule.getAtom(icent).charge );
     *mult = 1; /* no atom is multiplied by a symmetry factor */
 }
 
@@ -575,7 +575,7 @@ ShellTree::getOverlappingWith(const real *center, real cellsz,
 
 /** Ther standard constructor. */
 ErgoMolInfo::ErgoMolInfo(const BasisInfoStruct& bis_, const Molecule& mol)
-    : GridGenMolInfo(mol.noOfAtoms,bis_.noOfBasisFuncs, bis_.noOfShells),
+  : GridGenMolInfo(mol.getNoOfAtoms(),bis_.noOfBasisFuncs, bis_.noOfShells),
       bis(bis_), molecule(mol), shellTree(NULL)
 { 
     ergo_real *rShell = new ergo_real[bis.noOfShells];
@@ -651,7 +651,7 @@ ErgoMolInfo::getBlocks(const real *center, real cellsz, const real *rshell,
 void
 ErgoMolInfo::getExps(int *maxl, int **bascnt, real (**aa)[2]) const
 {
-    int icent, ishell, il, lda, noOfAtoms = molecule.noOfAtoms;
+  int icent, ishell, il, lda, noOfAtoms = molecule.getNoOfAtoms();
     real (*la)[2];
     const Molecule& ms = molecule;
     
@@ -678,12 +678,12 @@ ErgoMolInfo::getExps(int *maxl, int **bascnt, real (**aa)[2]) const
 	/* Go through all atoms to check if this shell is sitting 
 	   on an atom center */
         for(icent=0; icent<noOfAtoms; icent++) {
-            real dx = shell.centerCoords[0]-ms.atoms[icent].coords[0];
-            real dy = shell.centerCoords[1]-ms.atoms[icent].coords[1];
-            real dz = shell.centerCoords[2]-ms.atoms[icent].coords[2];
-            
-            if(dx*dx + dy*dy+dz*dz < 1e-10)
-                break;
+	  real dx = shell.centerCoords[0]-ms.getAtom(icent).coords[0];
+	  real dy = shell.centerCoords[1]-ms.getAtom(icent).coords[1];
+	  real dz = shell.centerCoords[2]-ms.getAtom(icent).coords[2];
+	  // FIXME hard-coded constant 1e-10 here, is that bad?
+	  if(dx*dx + dy*dy+dz*dz < 1e-10)
+	    break;
         }
 	/* If (icent == noOfAtoms) that means this shell is not on any 
 	   atom (it is a "ghost" shell).
