@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -77,6 +77,8 @@
 #include "ci.h"
 #include "license.h"
 #include "xyz_file_parser.h"
+#include "electron_dynamics.h"
+#include "tdhf_dynamics.h"
 
 static void variable_free(struct variable* v);
 
@@ -90,6 +92,7 @@ public:
   struct variable* var_list;
   struct variable* J_K_params;
   struct variable* lr_params;
+  struct variable* ed_params;
   struct variable* mat_params;
   struct variable* scf_params;
   struct variable* XC_params;
@@ -106,6 +109,7 @@ public:
   JK::Params   jkOptions;
   SCF::Options    scfOptions;
   SCF::MatOptions matOptions;
+  ED::Params edOptions; /* Electron dynamics (ED) options. */
 
   void registerInputVariables();
   char *Basis;            /**< name of the current basis set. */
@@ -340,6 +344,7 @@ var_get_string(struct variable *root, const char *name)
 #define var_get_int(n)    var_get_int_template(NULL,       (n))
 #define var_get_intJK(n)  var_get_int_template(ergo.J_K_params, (n))
 #define var_get_intLR(n)  var_get_int_template(ergo.lr_params, (n))
+#define var_get_intED(n)  var_get_int_template(ergo.ed_params, (n))
 #define var_get_intMA(n)  var_get_int_template(ergo.mat_params, (n))
 #define var_get_intOU(n)  var_get_int_template(ergo.output_params, (n))
 #define var_get_intSCF(n) var_get_int_template(ergo.scf_params, (n))
@@ -347,11 +352,13 @@ var_get_string(struct variable *root, const char *name)
 #define var_get_real(n)    var_get_real_template(NULL,       (n))
 #define var_get_realJK(n)  var_get_real_template(ergo.J_K_params, (n))
 #define var_get_realLR(n)  var_get_real_template(ergo.lr_params, (n))
+#define var_get_realED(n)  var_get_real_template(ergo.ed_params, (n))
 #define var_get_realMA(n)  var_get_real_template(ergo.mat_params, (n))
 #define var_get_realOU(n)  var_get_real_template(ergo.output_params, (n))
 #define var_get_realSCF(n) var_get_real_template(ergo.scf_params, (n))
 #define var_get_realXC(n)  var_get_real_template(ergo.XC_params, (n))
 #define var_get_stringLR(n)  var_get_string(ergo.lr_params, (n))
+#define var_get_stringED(n)  var_get_string(ergo.ed_params, (n))
 #define var_get_stringSCF(n) var_get_string(ergo.scf_params, (n))
 #define var_get_stringOU(n)  var_get_string(ergo.output_params, (n))
 #define var_get_stringXC(n)  var_get_string(ergo.XC_params, (n))
@@ -389,8 +396,8 @@ es_mol_begin(enum MolType moleculeClass) {
   ergo.moleculeUnit = 1;
   ergo.readingMoleculeClass = moleculeClass;
   switch(moleculeClass) {
-  case MOL_MAIN:  ergo.molecule.noOfAtoms = 0;      break;
-  case MOL_GHOST: ergo.ghostMolecule.noOfAtoms = 0; break;
+  case MOL_MAIN:  ergo.molecule.clear();      break;
+  case MOL_GHOST: ergo.ghostMolecule.clear(); break;
   default: assert(0);
   }
   if(Basis_info) {
@@ -412,21 +419,16 @@ es_add_atom(const char *name, double x, double y, double z)
   case MOL_GHOST: m = &ergo.ghostMolecule; break;
   default: assert(0);
   }
-  if(m->noOfAtoms == Molecule::MAX_NO_OF_ATOMS) {
-    do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "Too many atoms in %s molecule\n",
-              ergo.readingMoleculeClass == MOL_GHOST ? "ghost" : "main");
-    exit(1);
-  }
-  m->atoms[m->noOfAtoms].charge = get_charge_int_from_atom_label(name);
-  m->atoms[m->noOfAtoms].coords[0] = x*ergo.moleculeUnit;
-  m->atoms[m->noOfAtoms].coords[1] = y*ergo.moleculeUnit;
-  m->atoms[m->noOfAtoms].coords[2] = z*ergo.moleculeUnit;
+  ergo_real charge = get_charge_int_from_atom_label(name);
+  m->addAtom(charge,
+	     x*ergo.moleculeUnit,
+	     y*ergo.moleculeUnit,
+	     z*ergo.moleculeUnit);
   do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "%s (charge=%3.1f) %f %f %f", name,
-	    (double)m->atoms[m->noOfAtoms].charge,
+	    (double)charge,
             (double)(x*ergo.moleculeUnit), 
 	    (double)(y*ergo.moleculeUnit), 
 	    (double)(z*ergo.moleculeUnit));
-  m->noOfAtoms++; 
 }
 
 /** Finish the inline definition of the molecule. */
@@ -436,7 +438,7 @@ es_mol_commit(void)
   printf("Inline %s molecule with %d atoms.\n", 
          ergo.readingMoleculeClass == MOL_GHOST ? "ghost" : "main",
          ergo.readingMoleculeClass  == MOL_GHOST
-         ? ergo.ghostMolecule.noOfAtoms : ergo.molecule.noOfAtoms);
+         ? ergo.ghostMolecule.getNoOfAtoms() : ergo.molecule.getNoOfAtoms());
 }
 
 /** Selects the units for the inline molecule format to be Angtroms,
@@ -644,7 +646,7 @@ es_run(const char *mode, int save_pot)
   do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "running %s\n", mode);
 
   if(!ergoIntegralInfo)
-    ergoIntegralInfo = new IntegralInfo;
+    ergoIntegralInfo = new IntegralInfo(true);
   
   if(var_get_int("enable_memory_usage_output"))
     enable_memory_usage_output();
@@ -660,7 +662,7 @@ es_run(const char *mode, int save_pot)
   if (ergo.molecule.getNumberOfElectrons() <= 0) {
       do_output(LOG_CAT_ERROR, LOG_AREA_MAIN,
                 "es_run: no electrons found. Number of atoms: %d",
-                ergo.molecule.noOfAtoms);
+                ergo.molecule.getNoOfAtoms());
       return -1;
   }
 
@@ -687,7 +689,7 @@ es_run(const char *mode, int save_pot)
 	return -1;
       }
 
-    if(ergo.ghostMolecule.noOfAtoms>0 &&
+    if(ergo.ghostMolecule.getNoOfAtoms() > 0 &&
        Basis_info->addBasisfuncsForMolecule(ergo.ghostMolecule, 
                                             ergo.GhostBasis,
                                             Ergo::NO_OF_BASIS_SET_RANGES,
@@ -744,9 +746,11 @@ es_run(const char *mode, int save_pot)
   scf.create_basis_func_coord_file = var_get_intSCF("create_basis_func_coord_file");
   scf.output_homo_and_lumo_eigenvectors = var_get_intSCF("output_homo_and_lumo_eigenvectors");
   scf.create_mtx_file_S           = var_get_intSCF("create_mtx_file_S");
+  scf.create_mtx_file_H_core      = var_get_intSCF("create_mtx_file_H_core");
   scf.create_mtx_files_F          = var_get_intSCF("create_mtx_files_F");
   scf.create_mtx_files_D          = var_get_intSCF("create_mtx_files_D");
   scf.create_mtx_files_dipole     = var_get_intSCF("create_mtx_files_dipole");
+  scf.create_2el_integral_m_file  = var_get_intSCF("create_2el_integral_m_file");
   scf.force_restricted            = var_get_intSCF("force_restricted");
   scf.force_unrestricted          = var_get_intSCF("force_unrestricted");
   scf.max_no_of_diis_matrices     = var_get_intSCF("max_no_of_diis_matrices");
@@ -755,12 +759,12 @@ es_run(const char *mode, int save_pot)
   scf.max_number_of_iterations    = var_get_intSCF("max_number_of_iterations");
   //  scf.do_f_thresh_verification    = var_get_intSCF("do_f_thresh_verification");
   //  scf.do_comparison_to_simple_purification   = var_get_intSCF("do_comparison_to_simple_purification");
+  //  scf.do_puri_mmul_tests          = var_get_intSCF("do_puri_mmul_tests");
   //  scf.output_statistics_mfiles    = var_get_intSCF("output_statistics_mfiles");
   //  scf.do_sparsity_investigation   = var_get_intSCF("do_sparsity_investigation");
   //  scf.do_sparsity_investigation_reppuri   = var_get_intSCF("do_sparsity_investigation_reppuri");
   //  scf.sparsity_plots_resolution_r = var_get_intSCF("sparsity_plots_resolution_r");
   //  scf.sparsity_plots_resolution_m = var_get_intSCF("sparsity_plots_resolution_m");
-  scf.do_electron_dynamics        = var_get_intSCF("do_electron_dynamics");
   scf.no_of_careful_first_scf_steps     = var_get_intSCF("no_of_careful_first_scf_steps");
   scf.do_report_density_diff      = var_get_intSCF("do_report_density_diff");
   scf.no_of_impr_req_for_diis     = var_get_intSCF("no_of_impr_req_for_diis");
@@ -772,6 +776,10 @@ es_run(const char *mode, int save_pot)
   scf.use_diag_on_error           = var_get_intSCF("use_diag_on_error");
   scf.use_diag_on_error_guess     = var_get_intSCF("use_diag_on_error_guess");
   scf.output_mulliken_pop         = var_get_intSCF("output_mulliken_pop");
+  scf.output_density_images       = var_get_intSCF("output_density_images");
+  scf.output_density_images_only  = var_get_intSCF("output_density_images_only");
+  scf.output_density_images_boxwidth = var_get_realSCF("output_density_images_boxwidth");
+  scf.compute_gradient_fixeddens  = var_get_intSCF("compute_gradient_fixeddens");
   //  scf.do_acc_scan_J               = var_get_intSCF("do_acc_scan_J");
   //  scf.do_acc_scan_K               = var_get_intSCF("do_acc_scan_K");
   //  scf.do_acc_scan_Vxc             = var_get_intSCF("do_acc_scan_Vxc");
@@ -790,7 +798,16 @@ es_run(const char *mode, int save_pot)
 
   jkparams_set_from_vars(ergo.jkOptions);
 
-  ergo.molecule.netCharge  = var_get_int("charge");
+  ED::Params& ed = ergo.edOptions;
+  ed.max_time = var_get_realED("max_time");
+  ed.timestep = var_get_realED("timestep");
+  ed.dc_pulse_strength = var_get_realED("dc_pulse_strength");
+  ed.dc_pulse_time = var_get_realED("dc_pulse_time");
+  ed.ac_pulse_max = var_get_realED("ac_pulse_max");
+  ed.ac_pulse_omega = var_get_realED("ac_pulse_omega");
+  ed.field_type = var_get_stringED("field_type");
+
+  ergo.molecule.setNetCharge(var_get_int("charge"));
   int alpha_beta_diff = var_get_int("spin_polarization");
   if( (ergo.molecule.getNumberOfElectrons()-alpha_beta_diff)%2) {
       do_output(LOG_CAT_ERROR, LOG_AREA_MAIN,
@@ -816,6 +833,7 @@ es_run(const char *mode, int save_pot)
     var_get_intMA("sparse_matrix_block_factor_3");
   ergo.matOptions.threads = var_get_intMA("threads");
   ergo.matOptions.parallelLevel = var_get_intMA("parallelLevel");
+  ergo.matOptions.no_of_buffers_per_allocator = var_get_intMA("no_of_buffers_per_allocator");
   ergo.matOptions.prepare(*Basis_info);
 
   const char *tmpdir = var_get_string(NULL, "tmpdir");
@@ -896,8 +914,8 @@ es_run(const char *mode, int save_pot)
   /* Set charges in extraChargesMolecule according to parameters "extra_charges_mol_charge_h", "extra_charges_mol_charge_o" etc. */
   ergo_real extra_charges_atom_charge_h = var_get_real("extra_charges_atom_charge_h");
   ergo_real extra_charges_atom_charge_o = var_get_real("extra_charges_atom_charge_o");
-  for(int i = 0; i < ergo.extraChargesMolecule.noOfAtoms; i++) {
-    Atom & atom = ergo.extraChargesMolecule.atoms[i];
+  for(int i = 0; i < ergo.extraChargesMolecule.getNoOfAtoms(); i++) {
+    Atom atom = ergo.extraChargesMolecule.getAtom(i);
     if(atom.charge == 1)
       atom.charge = extra_charges_atom_charge_h;
     else if(atom.charge == 8)
@@ -907,6 +925,7 @@ es_run(const char *mode, int save_pot)
 		"Error processing extraChargesMolecule: only O and H atoms are supported. Found atom with charge %5.2f", atom.charge);
       return -1;
     }
+    ergo.extraChargesMolecule.replaceAtom(i, atom);
   }
 
   std::auto_ptr<BasisInfoStruct> basisInfoDensFit(new BasisInfoStruct(0)); /* FIXME */
@@ -970,7 +989,7 @@ es_run(const char *mode, int save_pot)
 	CI::Options ci_options;
 	// Call CI routine.
 	if(do_CI(*Basis_info,
-		 &*ergoIntegralInfo,
+		 *ergoIntegralInfo,
 		 ci_options,
 		 &S[0],
 		 &H_1[0],
@@ -1002,6 +1021,38 @@ es_run(const char *mode, int save_pot)
                          ergo.matOptions,
 			 threshold_integrals_1el);
       SCF.do_SCF_iterations();
+      // Optionally do electron dynamics calculation. */
+      int do_electron_dynamics_after_scf = var_get_int("do_electron_dynamics_after_scf");
+      if(do_electron_dynamics_after_scf == 1) {
+	// Get stuff from SCF object.
+	symmMatrix S_matrix;
+	SCF.get_overlap_matrix(S_matrix);
+	symmMatrix H_core;
+	SCF.get_H_core_matrix(H_core);
+	symmMatrix FockMatrix;
+	SCF.get_Fock_matrix(FockMatrix);
+	triangMatrix invCholFactor;
+	SCF.get_invCholFactor_matrix(invCholFactor);
+	symmMatrix densityMatrix;
+	SCF.get_density_matrix(densityMatrix);
+	JK::ExchWeights CAM_params;
+	get_hf_weight_and_cam_params(scf.use_dft, &CAM_params.alpha,
+				     &CAM_params.beta, &CAM_params.mu);
+	CAM_params.computeRangeSeparatedExchange = CAM_params.beta != ergo_real(0.0);
+	do_tdhf_dynamics(*Basis_info,
+			 *ergoIntegralInfo,
+			 ergo.molecule,
+			 ergo.extraChargesMolecule,
+			 ergo.matOptions,
+			 CAM_params,
+			 ergo.jkOptions,
+			 FockMatrix,
+			 densityMatrix,
+			 S_matrix,
+			 invCholFactor,
+			 ed);
+      }
+
     }
   } 
   catch (const std::bad_alloc & e) {
@@ -1075,7 +1126,7 @@ public:
     jkparams_set_from_vars(ergo.jkOptions);
     do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "calling compute_J_by_boxes_nosymm");
     if(compute_J_by_boxes_nosymm(*bi,
-				 ergoIntegralInfo,
+				 *ergoIntegralInfo,
 				 ergo.jkOptions,
 				 fmat,
 				 dmat) != 0)
@@ -1093,7 +1144,7 @@ public:
 	int symmetryFlag = 0;
 
 	if(compute_K_by_boxes(*bi,
-			      ergoIntegralInfo,
+			      *ergoIntegralInfo,
 			      CAM_params,
 			      ergo.jkOptions,
 			      K,
@@ -1452,6 +1503,8 @@ Ergo::registerInputVariables()
   KWMAT(sparse_matrix_block_factor_1, VAR_INT,"Block size factor determining block size at second lowest level: sparse_matrix_block_size * sparse_matrix_block_factor_1.");
   KWMAT(sparse_matrix_block_size,     VAR_INT,"Lowest level submatrix block size.");
   KWMAT(parallelLevel,                VAR_INT,"Level in matrix hierarchy for parallelization.");
+  KWMAT(no_of_buffers_per_allocator,  VAR_INT,"Number of buffers per allocator to use in matrix memory allocation manager. A large value means that large chunks of memory will be allocated at a time.");
+#undef KWMAT
 #undef KL
 
 #define KL ergo.lr_params
@@ -1459,10 +1512,22 @@ Ergo::registerInputVariables()
   KW(KL,convergence_threshold,       VAR_FLOAT,  9e-4  ,"Convergence threshold for linear response calculations.");
 #undef KL
 
+#define KL ergo.ed_params
+#define KWED(n,type,d) KW(ergo.ed_params,n,type,edOptions.n,d)
+  KW(ergo.ed_params,electron_dynamics_field_type,   VAR_STRING, "none", "Field type to use in electron dynamics simulation: \"none\", \"dc-pulse\", \"ac-pulse\".");
+  KWED(max_time, VAR_FLOAT, "How long time to run electron dynamics. Unit: a.u.");
+  KWED(timestep, VAR_FLOAT, "Timestep to use in electron dynamics. Unit: a.u.");
+  KWED(dc_pulse_strength, VAR_FLOAT, "Strength of DC pulse to use in electron dynamics. Unit: a.u.");
+  KWED(dc_pulse_time, VAR_FLOAT, "Duration of DC pulse to use in electron dynamics. Unit: a.u.");
+  KWED(ac_pulse_max, VAR_FLOAT, "Maximum field strength E_max for AC pulse to use in electron dynamics. Unit: a.u.");
+  KWED(ac_pulse_omega, VAR_FLOAT, "Frequency omega for AC pulse to use in electron dynamics. Unit: a.u.");
+#undef KWED
+#undef KL
+
 #define KL ergo.scf_params
 #define KWSCF(n,type,d) KW(ergo.scf_params,n,type,scfOptions.n,d)
   //  KWSCF(write_guess_density_only,     VAR_INT,"Only generate starting guess density, write it to file and exit.");
-  //  KWSCF(skip_H_core,                  VAR_INT,"Skip computation of 1-elerctron (core) Hamiltonian matrix. This gives bogus results, only useful if looking at results of initialization part or first cycle where H_core does not matter yet.");
+  //  KWSCF(skip_H_core,                  VAR_INT,"Skip computation of 1-electron (core) Hamiltonian matrix. This gives bogus results, only useful if looking at results of initialization part or first cycle where H_core does not matter yet.");
   KWSCF(use_simple_dense_H_core,      VAR_INT,"Use simple dense matrix computation of 1-electron (core) Hamiltonian matrix.");
   KWSCF(use_diis_always,              VAR_INT,"Always use DIIS even if the energy goes up.");
   KWSCF(write_overlap_matrix,         VAR_INT,"Write overlap matrix to file.");
@@ -1473,11 +1538,15 @@ Ergo::registerInputVariables()
   //  KWSCF(scan_start_thresh,            VAR_FLOAT,"Scan start threshold value to use for \"accuracy scans\" of J, K, Vxc matrices.");
   //  KWSCF(scan_step_factor,             VAR_FLOAT,"Step factor to use for \"accuracy scans\" of J, K, Vxc matrices.");
   //  KWSCF(purification_create_m_files,  VAR_INT,"Create m files with information about purification."),
-    //  KWSCF(purification_use_rand_perturbation_for_alleigsint, VAR_INT,"Use random perturbation to attempt getting easier Lanczos convergence when determining min/max eigenvalues of Fock matrix before purification."),
+  //  KWSCF(purification_use_rand_perturbation_for_alleigsint, VAR_INT,"Use random perturbation to attempt getting easier Lanczos convergence when determining min/max eigenvalues of Fock matrix before purification."),
   KWSCF(use_diagonalization,          VAR_INT,"Use diagonalization instead of purification.");
   KWSCF(use_diag_on_error,            VAR_INT,"Use diagonalization if purification fails.");
   KWSCF(use_diag_on_error_guess,      VAR_INT,"Use diagonalization if purification fails during starting guess projection.");
   KWSCF(output_mulliken_pop,          VAR_INT,"Output Mulliken population analysis stuff (atomic charges and atomic spin densities) after SCF finished.");
+  KWSCF(output_density_images,        VAR_INT,"Output density image files (including Gabedit gcube density files) after SCF finished.");
+  KWSCF(output_density_images_only,   VAR_INT,"Output density image files (including Gabedit gcube density files) directly, without any SCF procedure, using the given starting guess density.");
+  KWSCF(output_density_images_boxwidth, VAR_FLOAT, "Box width to use for density image files (including Gabedit gcube density files).");
+  KWSCF(compute_gradient_fixeddens,   VAR_INT,"Compute gradient of energy with respect to nuclear positions for fixed electron density.");
   KWSCF(step_length_start,            VAR_FLOAT,"Start value of SCF step length parameter.");
   KWSCF(step_length_giveup,           VAR_FLOAT,"When step length in SCF procedure becomes smaller than this value, give up.");
   KWSCF(starting_guess_disturbance,   VAR_FLOAT,"Magnitude of random disturbance to apply to starting guess density matrix.");
@@ -1490,12 +1559,12 @@ Ergo::registerInputVariables()
   KWSCF(max_number_of_iterations,     VAR_INT,"Maximum number of SCF iterations.");
   //  KWSCF(do_f_thresh_verification,     VAR_INT,"When truncating Fock matrix, verify that the error matrix norm is below the requested threshold.");
   //  KWSCF(do_comparison_to_simple_purification,     VAR_INT,"In each density matrix construction, do a comparison to a simple purification scheme.");
+  //  KWSCF(do_puri_mmul_tests,           VAR_INT,"In each density matrix construction, do some extra purification test runs comparing different matrix-matrix multiplication and truncation approaches.");
   //  KWSCF(output_statistics_mfiles,     VAR_INT,"Output m-file with statistics (timings etc) each SCF iteration.");
   //  KWSCF(do_sparsity_investigation,    VAR_INT,"Create distance-vs-magnitude plot files.");
   //  KWSCF(do_sparsity_investigation_reppuri,    VAR_INT,"Perform purification 3 times using different norms, and create corresponding distance-vs-magnitude plot files.");
   //  KWSCF(sparsity_plots_resolution_r,  VAR_INT,"Distance resolution parameter for sparsity plots.");
   //  KWSCF(sparsity_plots_resolution_m,  VAR_INT,"Magnitude resolution parameter for sparsity plots.");
-  KWSCF(do_electron_dynamics,         VAR_INT,"Perform TDHF electron dynamics calculation after SCF.");
   KWSCF(no_of_careful_first_scf_steps,VAR_INT,"Number of \"careful\" steps in beginning of SCF procedure. May be useful if starting guess is very bad.");
   KWSCF(do_report_density_diff,       VAR_INT,"In each SCF cycle, output norm of difference between current and previous density matrix.");
   KWSCF(max_no_of_diis_matrices,      VAR_INT,"Max number of DIIS matrices to use. Lower value may be useful to reduce disk/memory usage.");
@@ -1519,12 +1588,14 @@ Ergo::registerInputVariables()
   KWSCF(create_basis_func_coord_file, VAR_INT,"Write text file with coordinates of basis functions.");
   KWSCF(output_homo_and_lumo_eigenvectors, VAR_INT,"Output HOMO and LUMO molecular orbital coefficients.");
   KWSCF(create_mtx_file_S,            VAR_INT,"Write overlap matrix to file in matrix market format (mtx).");
+  KWSCF(create_mtx_file_H_core,       VAR_INT,"Write core Hamiltonian matrix to file in matrix market format (mtx).");
   KWSCF(create_mtx_files_F,           VAR_INT,"Write effective Hamiltonian matrices to file in matrix market format (mtx).");
   KWSCF(create_mtx_files_D,           VAR_INT,"Write density matrices to file in matrix market format (mtx).");
   KWSCF(create_mtx_files_dipole,      VAR_INT,"Write dipole matrices to file in matrix market format (mtx).");
 
   KW(ergo.scf_params,calculation_identifier,   VAR_STRING, "N/A","String to identify the calculation, used for example in mtx-files.");
 
+  KWSCF(create_2el_integral_m_file,   VAR_INT,"Create m-file with rank-4 tensor containing the values of all 2-electron integrals. Very large; O(N^4); only use for small cases.");
 
 #undef KWSCF
 #undef KL
@@ -1538,6 +1609,7 @@ Ergo::registerInputVariables()
   KW(KL,ghost_basis,                 VAR_STRING, ""    ,"Basis set to use for \"ghost molecule\".");
   KW(KL,enable_memory_usage_output,  VAR_INT,    0,"Write information about memory usage to output file.");
   KW(KL,do_ci_after_scf,             VAR_INT,    0,"Perform a CI (configuration interaction) calculation after the SCF calculation.");
+  KW(KL,do_electron_dynamics_after_scf,             VAR_INT,    0,"Perform electron dynamics (e.g. TDHF) calculation after the SCF calculation.");
   KW(KL,use_6_d_functions,           VAR_INT,    0,"If set to 1, use 6 basis functions in d-type shells instead of the usual 5 functions.");
   KW(KL,rand_seed,                   VAR_INT,    0,"Random seed initializing sequence of random numbers.");
   KW(KL,charge,                      VAR_INT,    0,"Net charge of molecule; this number (together with the nuclei) determines the number of electrons in the system.");
@@ -1550,6 +1622,7 @@ Ergo::registerInputVariables()
   //KW(KL,output,                      VAR_LIST,   ergo.output_params,"List of input variables related to output.");
   KW(KL,mat,                         VAR_LIST,   ergo.mat_params,"List of input variables related to hierarchic matrix library.");
   KW(KL,lr,                          VAR_LIST,   ergo.lr_params,"List of input variables related to linear response calculations.");
+  KW(KL,ed,                          VAR_LIST,   ergo.ed_params,"List of input variables related to electron dynamics calculations.");
   KW(KL,XC,                          VAR_LIST,   ergo.XC_params,"List of input variables related to the computation of the DFT exchange-correlation contribution.");
   KW(KL,J_K,                         VAR_LIST,   ergo.J_K_params,"List of input variables related to the computation of Coulomb and HF exchange matrices.");
 
@@ -1610,6 +1683,7 @@ int
 main(int argc, char *argv[])
 {
   try {
+    enable_output();
   do_output(LOG_CAT_INFO, LOG_AREA_MAIN, ERGO_LICENSE_TEXT_LONG);
   bool executed_something = false;
   FILE *inp_file = NULL;
@@ -1656,6 +1730,21 @@ main(int argc, char *argv[])
         return 1;
       }
       i++;
+    } else if(strcmp(argv[i], "-g") == 0) {
+      if(i+1<argc) {
+        do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "Reading ghost molecule from file: %s\n", argv[i+1]);
+	if(readMoleculeFileInXyzFormat(ergo.ghostMolecule, 
+				       argv[i+1], 
+				       0, /* This param has no meaning in this case. */
+				       false) != 0) {
+	  do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "readMoleculeFileInXyzFormat() for ghost molecule failed.");
+	  return 1;
+	}
+      } else {
+        fprintf(stderr, "option -g encountered without argument\n");
+        return 1;
+      }
+      i++;
     } else if(strcmp(argv[i], "-h") == 0) {
       printf("Usage: ergo [args...]\n"
              "args can be: input file name\n"
