@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@
 
 
 static ergo_real
-get_M(const IntegralInfo* integralInfo,
+get_M(const IntegralInfo & integralInfo,
       const BasisInfoStruct & basisInfo)
 {
   const JK::ExchWeights CAM_params_not_used;
@@ -112,7 +112,7 @@ compute_extent_for_all_basis_funcs_core(const BasisInfoStruct & basisInfo,
 
 
 int
-compute_extent_for_all_basis_funcs_2el(const IntegralInfo* integralInfo,
+compute_extent_for_all_basis_funcs_2el(const IntegralInfo & integralInfo,
 				       const BasisInfoStruct & basisInfo, 
 				       ergo_real* basisFuncExtentList, 
 				       ergo_real threshold,
