@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,13 +45,13 @@
 
 static int
 get_maxLimitingFactor(const BasisInfoStruct & basisInfo,
-		      const IntegralInfo* integralInfo,
+		      const IntegralInfo & integralInfo,
 		      const ergo_real* basisFuncExtentList,
 		      ergo_real* result_maxLimitingFactor,
 		      const BoxSystem & boxSystem,
 		      const box_item_struct* itemList)
 {
-  IntegratorWithMemory integrator(integralInfo);
+  IntegratorWithMemory integrator(&integralInfo);
   int n = basisInfo.noOfBasisFuncs;
   ergo_real maxExtent = 0;
   for(int i = 0; i < n; i++) {
@@ -109,12 +109,12 @@ get_maxLimitingFactor(const BasisInfoStruct & basisInfo,
 
 int
 get_basis_func_pair_list_2el(const BasisInfoStruct & basisInfo,
-			     const IntegralInfo* integralInfo,
+			     const IntegralInfo & integralInfo,
 			     ergo_real threshold,
 			     ergo_real maxDensityMatrixElement,
 			     std::vector<basis_func_index_pair_struct>  & resultList)
 {
-  IntegratorWithMemory integrator(integralInfo);
+  IntegratorWithMemory integrator(&integralInfo);
   int n = basisInfo.noOfBasisFuncs;
   
   do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "get_basis_func_pair_list, n = %6i", n);
