@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,10 +27,10 @@
 
 #include <stdlib.h>
 #include <cmath>
+#include <cstring>
 #include "boysfunction.h"
 #include "pi.h"
 #include "output.h"
-#include "polydegree.h"
 #include "utilities.h"
 
 
@@ -153,28 +153,13 @@ BoysFunction_raw_simpson(int n, ergo_real x)
 }
 
 
-/* We need Boys functions up to a degree 4 times the highest degree of basis functions. */
-#define BOYS_N_MAX (BASIS_FUNC_POLY_MAX_DEGREE*4+1)
 
-#define BOYS_TAB_DEGREE 12
-#define BOYS_X_MAX 160.0
-#define BOYS_NO_OF_INTERVALS 200
 
-typedef struct
+void
+BoysFunctionManager::init(void)
 {
-  ergo_real midx;
-  ergo_real A[BOYS_TAB_DEGREE];
-} BoysFuncIntervalStruct;
-
-BoysFuncIntervalStruct global_Boys_list[BOYS_N_MAX][BOYS_NO_OF_INTERVALS];
-int global_Boys_init_flag = 0;
-
-
-int
-boysfunction_init(void)
-{
-  if(global_Boys_init_flag == 1)
-    return 0;
+  if(Boys_init_flag == 1)
+    return;
   Util::TimeMeter timeMeter;
   ergo_real halfstep, kfactorial, BoysFuncRawResult, Ak, midx;
   halfstep = (ergo_real)BOYS_X_MAX / BOYS_NO_OF_INTERVALS * 0.5;
@@ -183,68 +168,46 @@ boysfunction_init(void)
       for(int j = 0; j < BOYS_NO_OF_INTERVALS; j++)
 	{
 	  midx = (ergo_real)BOYS_X_MAX * j / BOYS_NO_OF_INTERVALS + halfstep;
-	  global_Boys_list[n][j].midx = midx;
+	  Boys_list[n][j].midx = midx;
 	  kfactorial = 1;
 	  int minusOneToPowk = 1;
 	  for(int k = 0; k < BOYS_TAB_DEGREE; k++)
 	    {
 	      BoysFuncRawResult = BoysFunction_raw_simpson(n+k, midx);
 	      Ak = minusOneToPowk * BoysFuncRawResult / kfactorial;
-	      global_Boys_list[n][j].A[k] = Ak;
+	      Boys_list[n][j].A[k] = Ak;
 	      kfactorial *= k+1;
 	      minusOneToPowk *= -1;
 	    } /* END FOR k */
 	} /* END FOR j */
     } /* END FOR n */
-  global_Boys_init_flag = 1;
-  timeMeter.print(LOG_AREA_INTEGRALS, "boysfunction_init");
-  return 0;
+  Boys_init_flag = 1;
+  timeMeter.print(LOG_AREA_INTEGRALS, "BoysFunctionManager::init");
 }
 
-static ergo_real
-BoysFunction_pretabulated(int n, ergo_real x)
+ergo_real
+BoysFunctionManager::BoysFunction_pretabulated(int n, ergo_real x) const
 {
-  BoysFuncIntervalStruct* interval;
+  const BoysFuncIntervalStruct* interval;
   ergo_real intervalWidth, count, sum, deltax, deltaxtopowk;
   int intervalIndex, k;
-  if(global_Boys_init_flag != 1)
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in BoysFunction_pretabulated: "
-		"(global_Boys_init_flag != 1)\n");
-      exit(0);
-      return 0;
-    }
+  if(Boys_init_flag != 1)
+    throw std::runtime_error("Error in BoysFunctionManager::BoysFunction_pretabulated: (Boys_init_flag != 1).");
   if(x < 0)
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in BoysFunction_pretabulated: (x < 0)\n");
-      exit(0);
-      return 0;
-    }
+    throw std::runtime_error("Error in BoysFunctionManager::BoysFunction_pretabulated: (x < 0).");
   if(n >= BOYS_N_MAX)
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in BoysFunction_pretabulated: (n >= BOYS_N_MAX)\n");
-      exit(0);
-      return 0;
-    }
-  if(x >= BOYS_X_MAX)
-    {
-      /* use "large x formula" */
-      return (semiFactorial(2*n-1) / std::pow((ergo_real)2, n+1)) * std::sqrt(pi / std::pow(x, 2*n+1));
-    }
+    throw std::runtime_error("Error in BoysFunctionManager::BoysFunction_pretabulated: (n >= BOYS_N_MAX).");
+  if(x >= BOYS_X_MAX) {
+    /* use "large x formula" */
+    return (semiFactorial(2*n-1) / std::pow((ergo_real)2, n+1)) * std::sqrt(pi / std::pow(x, 2*n+1));
+  }
   /* choose which interval to use */
   intervalWidth = (ergo_real)BOYS_X_MAX / BOYS_NO_OF_INTERVALS;
   count = x / intervalWidth;
   intervalIndex = (int)std::floor(count);
   if((intervalIndex < 0) || (intervalIndex >= BOYS_NO_OF_INTERVALS))
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in BoysFunction_pretabulated: bad intervalIndex\n");
-      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "intervalIndex = %i", intervalIndex);
-      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "x = %22.11f", (double)x);
-      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "count = %22.11f", (double)count);
-      exit(0);
-      return 0;
-    }
-  interval = &global_Boys_list[n][intervalIndex];
+    throw std::runtime_error("Error in BoysFunctionManager::BoysFunction_pretabulated: bad intervalIndex.");
+  interval = &Boys_list[n][intervalIndex];
   sum = 0;
   deltax = x - interval->midx;
   deltaxtopowk = 1;
@@ -258,12 +221,39 @@ BoysFunction_pretabulated(int n, ergo_real x)
 }
 
 ergo_real
-BoysFunction(int n, ergo_real x)
-{
-#if 1
+BoysFunctionManager::BoysFunction(int n, ergo_real x) const {
   return BoysFunction_pretabulated(n, x);
-#else
-  return BoysFunction_raw(n, x);
-#endif
 }
 
+/** Function needed for Chunks&Tasks usage. */
+void BoysFunctionManager::write_to_buffer ( char * dataBuffer, size_t const bufferSize ) const {
+  char* p = dataBuffer;
+  if(bufferSize < get_size())
+    throw std::runtime_error("Error: bufferSize too small.");
+  // Boys_list
+  memcpy(p, Boys_list, sizeof(Boys_list));
+  p += sizeof(Boys_list);
+  // Boys_init_flag
+  memcpy(p, &Boys_init_flag, sizeof(int));
+  p += sizeof(int);
+  // DONE!  
+}
+
+/** Function needed for Chunks&Tasks usage. */
+size_t BoysFunctionManager::get_size() const {
+  return sizeof(Boys_list) + sizeof(int);
+}
+
+/** Function needed for Chunks&Tasks usage. */
+void BoysFunctionManager::assign_from_buffer ( char const * dataBuffer, size_t const bufferSize) {
+  const char* p = dataBuffer;
+  // Boys_list
+  memcpy(Boys_list, p, sizeof(Boys_list));
+  p += sizeof(Boys_list);
+  // Boys_init_flag
+  memcpy(&Boys_init_flag, p, sizeof(int));
+  p += sizeof(int);
+  // DONE!
+  if(static_cast<size_t>(p-dataBuffer) > bufferSize)
+    throw std::runtime_error("Error: (p > bufferSize).");  
+}
