@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -452,7 +452,7 @@ do_interaction_recursive(const IntegralInfo & integralInfo,
 	  ergo_real dx1 = currAtom->coords[1] - distr1->centerCoords[1];
 	  ergo_real dx2 = currAtom->coords[2] - distr1->centerCoords[2];
 	  ergo_real resultPreFactor = 2 * pi / alpha1;
-	  get_related_integrals_hermite(&integralInfo.monomial_info,
+	  get_related_integrals_hermite(integralInfo,
 					CAM_params_not_used,
 					n1, noOfMonomials_1,
 					n2, noOfMonomials_2,
@@ -462,11 +462,11 @@ do_interaction_recursive(const IntegralInfo & integralInfo,
 					alpha0,
 					resultPreFactor,
 					primitiveIntegralList_h);
-	  integralInfo.hermite_conversion_info.multiply_by_hermite_conversion_matrix_from_right(n1,
-												n2,
-												1.0/alpha1,
-												primitiveIntegralList_h,
-												primitiveIntegralList_2);	  
+	  integralInfo.multiply_by_hermite_conversion_matrix_from_right(n1,
+									n2,
+									1.0/alpha1,
+									primitiveIntegralList_h,
+									primitiveIntegralList_2);	  
 	  for(int id = 0; id < nDistrs; id++)
 	    {
 	      const DistributionSpecStruct* distr = &list[id].distr;
@@ -562,14 +562,13 @@ get_list_of_distrs_for_V(const BasisInfoStruct& basisInfo,
 }
 
 
-const ergo_real tolernance_dist = 1e-10;
-const ergo_real tolernance_exponent = 1e-11;
 
 static int
 compare_distrs(const void* p1, const void* p2)
 {
   DistributionSpecStructWithPairIdx* d1 = (DistributionSpecStructWithPairIdx*)p1;
   DistributionSpecStructWithPairIdx* d2 = (DistributionSpecStructWithPairIdx*)p2;
+  /* FIXME: Not nice to have these two hardcoded values here.  */
   const ergo_real tolernance_dist = 1e-10;
   const ergo_real tolernance_exponent = 1e-11;
   ergo_real dx = d1->distr.centerCoords[0] - d2->distr.centerCoords[0];
@@ -630,11 +629,11 @@ getSafeMaxDistance(const BasisInfoStruct& basisInfo, const Molecule& molecule)
 	    maxCoords[k] = x;
 	}
     }
-  for(int i = 0; i < molecule.noOfAtoms; i++)
+  for(int i = 0; i < molecule.getNoOfAtoms(); i++)
     {
       for(int k = 0; k < 3; k++)
 	{
-	  ergo_real x = molecule.atoms[i].coords[k];
+	  ergo_real x = molecule.getAtom(i).coords[k];
 	  if(x < minCoords[k])
 	    minCoords[k] = x;
 	  if(x > maxCoords[k])
@@ -668,9 +667,9 @@ compute_V_linear(const BasisInfoStruct& basisInfo,
 
   // Get maxCharge
   ergo_real maxCharge = 0;
-  for(int i = 0; i < molecule.noOfAtoms; i++)
+  for(int i = 0; i < molecule.getNoOfAtoms(); i++)
     {
-      ergo_real currCharge = molecule.atoms[i].charge;
+      ergo_real currCharge = molecule.getAtom(i).charge;
       if(currCharge > maxCharge)
 	maxCharge = currCharge;
     }
@@ -738,6 +737,8 @@ compute_V_linear(const BasisInfoStruct& basisInfo,
   ergo_real maxDistance = getSafeMaxDistance(basisInfo, molecule);
   // Add small margin to handle cases where maxDistance=0
   maxDistance += 0.1;
+  // Add some more margin that can be needed when multipoles end up somewhat further away when placed at box centers.
+  maxDistance += 5.0;
   mm_limits_init(maxDistance);
 
   // Note that boxList and atomListReordered are allocated by create_nuclei_mm_tree,
@@ -745,7 +746,7 @@ compute_V_linear(const BasisInfoStruct& basisInfo,
   atom_box_struct* boxList = NULL;
   Atom *atomListReordered = NULL;
   int numberOfLevels = -1;
-  if(create_nuclei_mm_tree(molecule.noOfAtoms, molecule.atoms, boxSize,
+  if(create_nuclei_mm_tree(molecule.getNoOfAtoms(), molecule.getAtomListPtr(), boxSize,
 			   &boxList, &numberOfLevels,
 			   &atomListReordered) != 0) 
     {
@@ -887,7 +888,7 @@ compute_V_linear(const BasisInfoStruct& basisInfo,
 static ergo_real 
 simplePrimVintegralSingle(DistributionSpecStruct* prim,
                           const Atom* atom,
-                          const IntegralInfo* integralInfo)
+                          const IntegralInfo & integralInfo)
 {
   ergo_real result2 = do_1e_repulsion_integral_using_symb_info(prim,
                                                                atom->charge,
@@ -902,7 +903,7 @@ simplePrimVintegral_list(DistributionSpecStruct* list,
                          int nPrims,
 			 const Atom* atom,
                          ergo_real threshold,
-                         const IntegralInfo* integralInfo)
+                         const IntegralInfo & integralInfo)
 {
   int k;
   ergo_real sum;
@@ -968,7 +969,7 @@ compute_V_matrix_full(const BasisInfoStruct& basisInfo,
                                                       nNewPrims,
                                                       &atomList[A],
                                                       threshold,
-                                                      &integralInfo);
+                                                      integralInfo);
                     } /* END FOR A */
                 } /* END FOR k */
             } /* END FOR j */
