@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -55,7 +55,7 @@ typedef struct
 
 static int 
 get_JK_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
-					 const IntegralInfo* integralInfo,
+					 const IntegralInfo & integralInfo,
 					 int maxNoOfMonomials,
 					 ergo_real* J,
 					 ergo_real* K,
@@ -196,8 +196,8 @@ get_JK_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
 			  ergo_real dz = groupList_2[group_j].centerCoords[2] - groupList_1[group_i].centerCoords[2];
 
 			  // now we have dx dy dz alpha0 alpha1 n1max n2max. Get all integrals for this case.
-			  int noOfMonomials_1 = integralInfo->monomial_info.no_of_monomials_list[n1max];
-			  int noOfMonomials_2 = integralInfo->monomial_info.no_of_monomials_list[n2max];
+			  int noOfMonomials_1 = integralInfo.monomial_info.no_of_monomials_list[n1max];
+			  int noOfMonomials_2 = integralInfo.monomial_info.no_of_monomials_list[n2max];
 
 			  if(get_related_integrals_h(integralInfo, 
 						     CAM_params_not_used,
@@ -361,7 +361,7 @@ typedef struct
 
 int
 compute_JK_single_box(const BasisInfoStruct & basisInfo,
-		      const IntegralInfo* integralInfo,
+		      const IntegralInfo & integralInfo,
 		      ergo_real* J,
 		      ergo_real* K,
 		      const ergo_real* dens,
@@ -465,7 +465,7 @@ compute_JK_single_box(const BasisInfoStruct & basisInfo,
       int degree = 0;
       for(int j = 0; j < 3; j++)
 	degree += distrList[i].distr.monomialInts[j];
-      int noOfMonomials = integralInfo->monomial_info.no_of_monomials_list[degree];
+      int noOfMonomials = integralInfo.monomial_info.no_of_monomials_list[degree];
       if(noOfMonomials > maxNoOfMonomials)
 	maxNoOfMonomials = noOfMonomials;
     } // END FOR ABcount
