@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 
 #include <cstring>
 #include <cstdio>
+#include <cassert>
 
 #include <pthread.h>
 
@@ -79,7 +80,7 @@ typedef struct
 
 static int 
 get_J_contribs_from_2_interacting_boxes_2(const BasisInfoStruct & basisInfo,
-					  const IntegralInfo* integralInfo,
+					  const IntegralInfo & integralInfo,
 					  int maxNoOfMonomials,
 					  ergo_real* result_J_list,
 					  const minimal_distr_struct* minimalDistrList_1,
@@ -188,8 +189,8 @@ get_J_contribs_from_2_interacting_boxes_2(const BasisInfoStruct & basisInfo,
 			  ergo_real dz = groupList_2[group_j].centerCoords[2] - groupList_1[group_i].centerCoords[2];
 
 			  // now we have dx dy dz alpha0 alpha1 n1max n2max. Get all integrals for this case.
-			  int noOfMonomials_1 = integralInfo->monomial_info.no_of_monomials_list[n1max];
-			  int noOfMonomials_2 = integralInfo->monomial_info.no_of_monomials_list[n2max];
+			  int noOfMonomials_1 = integralInfo.monomial_info.no_of_monomials_list[n1max];
+			  int noOfMonomials_2 = integralInfo.monomial_info.no_of_monomials_list[n2max];
 
 			  if(get_related_integrals_h(integralInfo, 
 						     CAM_params_not_used,
@@ -544,7 +545,7 @@ get_min_distance_from_point_to_box(const ergo_real* boxCenterCoords,
 
 static int
 get_joblists_J_for_two_boxes_recursive(const BasisInfoStruct & basisInfo,
-				       const IntegralInfo* integralInfo,
+				       const IntegralInfo & integralInfo,
 				       int maxNoOfMonomials,
 				       ergo_real threshold,
 				       const box_struct* boxList,
@@ -765,13 +766,13 @@ get_joblists_J_for_two_boxes_recursive(const BasisInfoStruct & basisInfo,
 
 static int
 get_list_of_labeled_distrs_maxLimitingFactor_linear(const BasisInfoStruct & basisInfo,
-						    const IntegralInfo* integralInfo,
+						    const IntegralInfo & integralInfo,
 						    ergo_real threshold,
 						    const basis_func_index_pair_struct* basisFuncIndexPairList,
 						    int basisFuncIndexPairCount,
 						    ergo_real* resultMaxLimitingFactor)
 { 
-  IntegratorWithMemory integrator(integralInfo);
+  IntegratorWithMemory integrator(&integralInfo);
 
   ergo_real maxLimitingFactor = 0;
   for(int kk = 0; kk < basisFuncIndexPairCount; kk++)
@@ -809,7 +810,7 @@ get_list_of_labeled_distrs_maxLimitingFactor_linear(const BasisInfoStruct & basi
 
 static int
 get_list_of_labeled_distrs_linear(const BasisInfoStruct & basisInfo,
-				  const IntegralInfo* integralInfo,
+				  const IntegralInfo & integralInfo,
 				  ergo_real threshold,
 				  DistributionSpecStructLabeled* resultList,
 				  int maxCountDistrs,
@@ -820,7 +821,7 @@ get_list_of_labeled_distrs_linear(const BasisInfoStruct & basisInfo,
 {
   ergo_real maxDensityMatrixElement = get_max_abs_vector_element(basisFuncIndexPairCount, D_list);
 
-  IntegratorWithMemory integrator(integralInfo);
+  IntegratorWithMemory integrator(&integralInfo);
 
   // create list of product primitives, with labels
   int distrCount = 0;
@@ -930,7 +931,7 @@ get_largest_and_smallest_extent_for_list_of_distributions(int n,
 static int
 get_max_no_of_monomials_for_list_of_distributions(int n, 
 						  const DistributionSpecStructLabeled* distrList, 
-						  const IntegralInfo* integralInfo)
+						  const IntegralInfo & integralInfo)
 {
   int maxNoOfMonomials = 0;
   for(int i = 0; i < n; i++)
@@ -938,7 +939,7 @@ get_max_no_of_monomials_for_list_of_distributions(int n,
       int degree = 0;
       for(int j = 0; j < 3; j++)
 	degree += distrList[i].distr.monomialInts[j];
-      int noOfMonomials = integralInfo->monomial_info.no_of_monomials_list[degree];
+      int noOfMonomials = integralInfo.monomial_info.no_of_monomials_list[degree];
       if(noOfMonomials > maxNoOfMonomials)
 	maxNoOfMonomials = noOfMonomials;
     } // END FOR ABcount
@@ -1098,7 +1099,7 @@ static int
 execute_joblist_J_std_serial(int noOfJobs_J_standard,
 			     const job_list_standard_entry_J_struct* jobList_J_standard,
 			     const BasisInfoStruct & basisInfo,
-			     const IntegralInfo* integralInfo,
+			     const IntegralInfo & integralInfo,
 			     int maxNoOfMonomials,
 			     ergo_real* result_J_list,
 			     const box_struct* boxList,
@@ -1188,7 +1189,7 @@ execute_joblist_J_std_thread_func(void* arg)
   J_std_joblist_thread_struct* params = (J_std_joblist_thread_struct*)arg;
 
   JK_contribs_buffer_struct bufferStruct;
-  allocate_buffers_needed_by_integral_code(params->integralInfo, params->maxNoOfMonomials, 0, &bufferStruct);
+  allocate_buffers_needed_by_integral_code(*params->integralInfo, params->maxNoOfMonomials, 0, &bufferStruct);
 
   const box_struct* boxList = params->boxList;
 
@@ -1205,7 +1206,7 @@ execute_joblist_J_std_thread_func(void* arg)
       if(boxIndex_1 == boxIndex_2 && branchIndex_1 == branchIndex_2)
 	self = 1;
       if(get_J_contribs_from_2_interacting_boxes_2(params->basisInfo,
-						   params->integralInfo,
+						   *params->integralInfo,
 						   params->maxNoOfMonomials,
 						   params->result_J_list,
 						  
@@ -1254,7 +1255,7 @@ execute_joblist_J_std_threaded(int noOfThreads,
 			       int noOfJobs_J_standard,
 			       const job_list_standard_entry_J_struct* jobList_J_standard,
 			       const BasisInfoStruct & basisInfo,
-			       const IntegralInfo* integralInfo,
+			       const IntegralInfo & integralInfo,
 			       int maxNoOfMonomials,
 			       ergo_real* result_J_list,
 			       int noOfBasisFuncIndexPairs,
@@ -1272,7 +1273,7 @@ execute_joblist_J_std_threaded(int noOfThreads,
     {
       threadParamsList[i] = new J_std_joblist_thread_struct(basisInfo);
       //threadParamsList[i].basisInfo = basisInfo;
-      threadParamsList[i]->integralInfo = integralInfo;
+      threadParamsList[i]->integralInfo = &integralInfo;
       threadParamsList[i]->maxNoOfMonomials = maxNoOfMonomials;
       threadParamsList[i]->boxList = boxList;
       threadParamsList[i]->jobList_J_standard = jobList_J_standard;
@@ -1846,7 +1847,7 @@ execute_joblist_J_fmm_thread(int noOfThreads, int noOfBasisFuncIndexPairs,
  */
 int
 compute_J_by_boxes_linear(const BasisInfoStruct & basisInfo,
-			  const IntegralInfo* integralInfo,
+			  const IntegralInfo & integralInfo,
 			  const JK::Params& J_K_params,
 			  const basis_func_index_pair_struct* basisFuncIndexPairList,
 			  int basisFuncIndexPairCount,
@@ -2120,9 +2121,9 @@ compute_J_by_boxes_linear(const BasisInfoStruct & basisInfo,
 			  ergo_real coeff = minimalDistrList[distrIndex].coeff;
 			  // get monomialInts from monomialIndex
 			  DistributionSpecStruct distr;
-			  distr.monomialInts[0] = integralInfo->monomial_info.monomial_list[monomialIndex].ix;
-			  distr.monomialInts[1] = integralInfo->monomial_info.monomial_list[monomialIndex].iy;
-			  distr.monomialInts[2] = integralInfo->monomial_info.monomial_list[monomialIndex].iz;
+			  distr.monomialInts[0] = integralInfo.monomial_info.monomial_list[monomialIndex].ix;
+			  distr.monomialInts[1] = integralInfo.monomial_info.monomial_list[monomialIndex].iy;
+			  distr.monomialInts[2] = integralInfo.monomial_info.monomial_list[monomialIndex].iz;
 			  distr.coeff = coeff;
 			  distr.exponent = currGroup->exponent;
 			  distr.centerCoords[0] = currGroup->centerCoords[0];
@@ -2130,7 +2131,7 @@ compute_J_by_boxes_linear(const BasisInfoStruct & basisInfo,
 			  distr.centerCoords[2] = currGroup->centerCoords[2];
 
 			  multipole_struct_small multipole;
-			  if(compute_multipole_moments(*integralInfo, &distr, &multipole) != 0)
+			  if(compute_multipole_moments(integralInfo, &distr, &multipole) != 0)
 			    {
 			      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in compute_multipole_moments");
 			      return -1;
@@ -2553,7 +2554,7 @@ compute_J_by_boxes_linear(const BasisInfoStruct & basisInfo,
   /* Execute multipole list */
   if(noOfThreads == 1)
     {
-      if( execute_joblist_J_fmm_serial(*integralInfo, basisInfo, J_K_params,
+      if( execute_joblist_J_fmm_serial(integralInfo, basisInfo, J_K_params,
                                        noOfJobs_J_multipole, &jobList_J_multipole[0],
                                        &boxList[0], maxnoOfMinimalDistrsPerBoxBranch,
                                        result_J_list) != 0)
@@ -2565,7 +2566,7 @@ compute_J_by_boxes_linear(const BasisInfoStruct & basisInfo,
   else
     {
       if( execute_joblist_J_fmm_thread(noOfThreads, noOfBasisFuncIndexPairs,
-                                       *integralInfo, basisInfo, J_K_params,
+                                       integralInfo, basisInfo, J_K_params,
                                        noOfJobs_J_multipole, &jobList_J_multipole[0],
                                        &boxList[0], maxnoOfMinimalDistrsPerBoxBranch,
                                        result_J_list) != 0)
@@ -2586,7 +2587,7 @@ compute_J_by_boxes_linear(const BasisInfoStruct & basisInfo,
 
 int
 compute_J_by_boxes(const BasisInfoStruct & basisInfo,
-		   const IntegralInfo* integralInfo,
+		   const IntegralInfo & integralInfo,
 		   const JK::Params& J_K_params,
 		   ergo_real* J,
 		   const ergo_real* dens)
@@ -2658,7 +2659,7 @@ but without assuming the density matrix to be symmetric.
 */
 int
 compute_J_by_boxes_nosymm(const BasisInfoStruct & basisInfo,
-			  const IntegralInfo* integralInfo,
+			  const IntegralInfo & integralInfo,
 			  const JK::Params& J_K_params,
 			  ergo_real* J,
 			  const ergo_real* dens)
