@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -118,7 +118,7 @@ pthread_mutex_t K_CSR_shared_access_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 static int 
 get_K_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
-					const IntegralInfo* integralInfo,
+					const IntegralInfo & integralInfo,
 					const JK::ExchWeights & CAM_params,
 					int maxNoOfMonomials,
 					ergo_real* K,
@@ -332,8 +332,8 @@ get_K_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
 			  // now we can do all integrals needed for this pair of groups
 
 			  // now we have dx dy dz alpha0 alpha1 n1max n2max. Get all integrals for this case.
-			  int noOfMonomials_1 = integralInfo->monomial_info.no_of_monomials_list[n1max];
-			  int noOfMonomials_2 = integralInfo->monomial_info.no_of_monomials_list[n2max];
+			  int noOfMonomials_1 = integralInfo.monomial_info.no_of_monomials_list[n1max];
+			  int noOfMonomials_2 = integralInfo.monomial_info.no_of_monomials_list[n2max];
 
 			  if(get_related_integrals_h(integralInfo, 
 						     CAM_params,
@@ -582,7 +582,7 @@ get_K_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
 
 static int
 create_joblist_exchange_for_two_boxes_recursive(const BasisInfoStruct & basisInfo,
-						const IntegralInfo* integralInfo,
+						const IntegralInfo & integralInfo,
 						int maxNoOfMonomials,
 						ergo_real threshold,
 						const box_struct* boxList,
@@ -764,7 +764,7 @@ execute_joblist_K_thread_func(void* arg)
   int noOfThreads = params->noOfThreads;
 
   JK_contribs_buffer_struct bufferStruct;
-  allocate_buffers_needed_by_integral_code(params->integralInfo, params->maxNoOfMonomials, params->basisFuncListCount_max, &bufferStruct);
+  allocate_buffers_needed_by_integral_code(*params->integralInfo, params->maxNoOfMonomials, params->basisFuncListCount_max, &bufferStruct);
 
   for(int jobIndex = 0; jobIndex < params->noOfJobs_K_total; jobIndex++)
     {
@@ -776,7 +776,7 @@ execute_joblist_K_thread_func(void* arg)
       if(boxIndex_1 == boxIndex_2)
 	self = 1;
       if(get_K_contribs_from_2_interacting_boxes(params->basisInfo,
-						 params->integralInfo,
+						 *params->integralInfo,
 						 params->CAM_params,
 						 params->maxNoOfMonomials,
 						 K,
@@ -869,7 +869,7 @@ execute_joblist_K_threaded(int noOfThreads,
 			   const ergo_real* dens,
 			   csr_matrix_struct* densCSR,
 			   const BasisInfoStruct & basisInfo,
-			   const IntegralInfo* integralInfo,
+			   const IntegralInfo & integralInfo,
 			   const JK::ExchWeights & CAM_params,
 			   int maxNoOfMonomials,
 			   int basisFuncListCount_max,
@@ -898,7 +898,7 @@ execute_joblist_K_threaded(int noOfThreads,
       threadParamsList[i] = new K_joblist_thread_struct(basisInfo, CAM_params);
       threadParamsList[i]->dens = dens;
       threadParamsList[i]->densCSR = densCSR;
-      threadParamsList[i]->integralInfo = integralInfo;
+      threadParamsList[i]->integralInfo = &integralInfo;
       threadParamsList[i]->maxNoOfMonomials = maxNoOfMonomials;
       threadParamsList[i]->basisFuncListCount_max = basisFuncListCount_max;
       threadParamsList[i]->boxList = boxList;
@@ -1028,7 +1028,7 @@ static int
 execute_joblist_K_serial(const ergo_real* dens,
                         csr_matrix_struct* densCSR,
 			 const BasisInfoStruct & basisInfo,
-			 const IntegralInfo* integralInfo,
+			 const IntegralInfo & integralInfo,
 			 const JK::ExchWeights & CAM_params,
 			 int maxNoOfMonomials,
 			 int basisFuncListCount_max,
@@ -1397,7 +1397,7 @@ This means that if only K is wanted, it must be set to zero before calling this
 */
 int
 compute_K_by_boxes(const BasisInfoStruct & basisInfo,
-		   const IntegralInfo* integralInfo,
+		   const IntegralInfo & integralInfo,
 		   const JK::ExchWeights & CAM_params_in,
 		   const JK::Params& J_K_params,
 		   ergo_real* K,
@@ -1502,7 +1502,7 @@ compute_K_by_boxes(const BasisInfoStruct & basisInfo,
       int degree = 0;
       for(int j = 0; j < 3; j++)
 	degree += distrList[i].distr.monomialInts[j];
-      int noOfMonomials = integralInfo->monomial_info.no_of_monomials_list[degree];
+      int noOfMonomials = integralInfo.monomial_info.no_of_monomials_list[degree];
       if(noOfMonomials > maxNoOfMonomials)
 	maxNoOfMonomials = noOfMonomials;
     } // END FOR ABcount
@@ -1630,7 +1630,7 @@ compute_K_by_boxes(const BasisInfoStruct & basisInfo,
       DistributionSpecStructLabeled* currDistr = &distrList[j];
       ergo_real* multipoleNormVectorList_curr = &multipoleNormVectorList[j*(MAX_MULTIPOLE_DEGREE_BASIC+1)];
       multipole_struct_small multipole;
-      if(compute_multipole_moments(*integralInfo, &currDistr->distr, &multipole) != 0)
+      if(compute_multipole_moments(integralInfo, &currDistr->distr, &multipole) != 0)
 	{
 	  do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in compute_multipole_moments");
 	  return -1;
@@ -1936,9 +1936,9 @@ compute_K_by_boxes(const BasisInfoStruct & basisInfo,
 		      ergo_real coeff = minimalDistrList[distrIndex].coeff;
 		      // get monomialInts from monomialIndex
 		      DistributionSpecStruct distr;
-		      distr.monomialInts[0] = integralInfo->monomial_info.monomial_list[monomialIndex].ix;
-		      distr.monomialInts[1] = integralInfo->monomial_info.monomial_list[monomialIndex].iy;
-		      distr.monomialInts[2] = integralInfo->monomial_info.monomial_list[monomialIndex].iz;
+		      distr.monomialInts[0] = integralInfo.monomial_info.monomial_list[monomialIndex].ix;
+		      distr.monomialInts[1] = integralInfo.monomial_info.monomial_list[monomialIndex].iy;
+		      distr.monomialInts[2] = integralInfo.monomial_info.monomial_list[monomialIndex].iz;
 		      distr.coeff = coeff;
 		      distr.exponent = currGroup->exponent;
 		      distr.centerCoords[0] = currGroup->centerCoords[0];
@@ -1946,7 +1946,7 @@ compute_K_by_boxes(const BasisInfoStruct & basisInfo,
 		      distr.centerCoords[2] = currGroup->centerCoords[2];
 
 		      multipole_struct_small multipole;
-		      if(compute_multipole_moments(*integralInfo, &distr, &multipole) != 0)
+		      if(compute_multipole_moments(integralInfo, &distr, &multipole) != 0)
 			{
 			  do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in compute_multipole_moments");
 			  return -1;
