@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -475,7 +475,7 @@ compare_distr_idxs_and_factor_structs(const void* p1in, const void* p2in) {
 
 int
 create_CSR_for_K(const BasisInfoStruct & basisInfo,
-		 const IntegralInfo* integralInfo,
+		 const IntegralInfo & integralInfo,
 		 const JK::Params& J_K_params,
 		 csr_matrix_struct* dens_CSR,
 		 csr_matrix_struct* K_CSR,
