@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -67,7 +67,7 @@ do_2e_integral(int mu,
 	       int la, 
 	       int si, 
 	       const BasisInfoStruct & basisInfo, 
-	       const IntegralInfo* integralInfo)
+	       const IntegralInfo & integralInfo)
 {
   int n_psi2, i, j;
   ergo_real sum, currIntegral;
@@ -132,7 +132,7 @@ do_2e_integral(int mu,
 */
 int 
 compute_2e_matrix_simple(const BasisInfoStruct & basisInfo,
-			 const IntegralInfo* integralInfo,
+			 const IntegralInfo & integralInfo,
                          ergo_real hf_weight,
 			 ergo_real* result,
 			 const ergo_real* dens)
@@ -171,7 +171,7 @@ compute_2e_matrix_simple(const BasisInfoStruct & basisInfo,
 
 static int 
 compute_J_and_K_integraldriven(const BasisInfoStruct & basisInfo,
-			       const IntegralInfo* integralInfo,
+			       const IntegralInfo & integralInfo,
 			       ergo_real* J,
 			       ergo_real* K,
 			       ergo_real* dens)
@@ -290,7 +290,7 @@ compute_J_and_K_integraldriven(const BasisInfoStruct & basisInfo,
 
 int
 compute_2e_matrix_list_explicit(const BasisInfoStruct & basisInfo,
-				const IntegralInfo* integralInfo,
+				const IntegralInfo & integralInfo,
 				ergo_real** resultList,
 				ergo_real** densList,
 				int noOfMatrices,
