@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@
 
 
 int compute_2e_matrix_list(const BasisInfoStruct & basisInfo,
-			   const IntegralInfo* integralInfo,
+			   const IntegralInfo & integralInfo,
 			   const JK::ExchWeights & CAM_params,
 			   ergo_real** resultList,
 			   ergo_real** densList,
@@ -43,7 +43,7 @@ int compute_2e_matrix_list(const BasisInfoStruct & basisInfo,
 			   const JK::Params& J_K_params);
 
 int compute_2e_matrix_list_difden(const BasisInfoStruct & basisInfo,
-				  const IntegralInfo* integralInfo,
+				  const IntegralInfo & integralInfo,
 				  const JK::ExchWeights & CAM_params,
 				  ergo_real** resultList,
 				  ergo_real** densList,
@@ -51,14 +51,14 @@ int compute_2e_matrix_list_difden(const BasisInfoStruct & basisInfo,
 				  const JK::Params& J_K_params);
 
 int compute_2e_matrix_exchange(const BasisInfoStruct & basisInfo,
-			       const IntegralInfo* integralInfo,
+			       const IntegralInfo & integralInfo,
 			       const JK::ExchWeights & CAM_params,
 			       ergo_real* K,
 			       ergo_real* dens,
 			       ergo_real threshold);
 
 int compute_2e_matrices_exchange(const BasisInfoStruct & basisInfo,
-				 const IntegralInfo* integralInfo,
+				 const IntegralInfo & integralInfo,
 				 const JK::ExchWeights & CAM_params,
 				 int noOfMatrices,
 				 ergo_real** K_list,
@@ -67,7 +67,7 @@ int compute_2e_matrices_exchange(const BasisInfoStruct & basisInfo,
 
 int compute_2e_matrix_coulomb(const BasisInfoStruct & basisInfo,
 			      const BasisInfoStruct & basisInfoDensFit,
-			      const IntegralInfo* integralInfo,
+			      const IntegralInfo & integralInfo,
 			      ergo_real* J,
 			      ergo_real* dens,
 			      const JK::Params& J_K_params,
