@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,5 +31,5 @@
 ergo_real do_2e_integral_using_symb_info(const JK::ExchWeights & CAM_params,
 					 const DistributionSpecStruct* psi1,
 					 const DistributionSpecStruct* psi2,
-					 const IntegralInfo* b);
+					 const IntegralInfo & integralInfo);
 
