@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -88,7 +88,7 @@ typedef struct
 ergo_real get_max_abs_vector_element(int n, const ergo_real* vector);
 
 void
-allocate_buffers_needed_by_integral_code(const IntegralInfo* integralInfo, 
+allocate_buffers_needed_by_integral_code(const IntegralInfo & integralInfo, 
 					 int maxNoOfMonomials,
 					 int basisFuncListCount_max,
 					 JK_contribs_buffer_struct* bufferStruct);
@@ -97,7 +97,7 @@ void
 free_buffers_needed_by_integral_code(JK_contribs_buffer_struct* bufferStruct);
 
 int
-get_related_integrals_h(const IntegralInfo* b,
+get_related_integrals_h(const IntegralInfo & integralInfo,
 			const JK::ExchWeights & CAM_params,
 			int n1max, int noOfMonomials_1,
 			int n2max, int noOfMonomials_2,
@@ -120,14 +120,14 @@ compute_extent_for_list_of_distributions(int n,
 
 int
 get_list_of_labeled_distrs_maxLimitingFactor(const BasisInfoStruct & basisInfo,
-					     const IntegralInfo* integralInfo,
+					     const IntegralInfo & integralInfo,
 					     ergo_real threshold,
 					     ergo_real* resultMaxLimitingFactor,
 					     ergo_real maxDensityMatrixElement);
 
 int
 get_list_of_labeled_distrs(const BasisInfoStruct & basisInfo,
-			   const IntegralInfo* integralInfo,
+			   const IntegralInfo & integralInfo,
 			   ergo_real threshold,
 			   DistributionSpecStructLabeled* resultList,
 			   int maxCountDistrs,
