@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 
 
 int
-get_related_integrals_hermite(const monomial_info_struct* monomial_info,
+get_related_integrals_hermite(const IntegralInfo & integralInfo,
 			      const JK::ExchWeights & paramsCAM,
 			      int n1max, int noOfMonomials_1,
 			      int n2max, int noOfMonomials_2,
@@ -66,14 +66,14 @@ get_related_integrals_hermite(const monomial_info_struct* monomial_info,
       /* Use downward recursion to get Boys function values */
       ergo_real arg1 =  alpha0 * R_12_squared;
       ergo_real expMinusArg1 = std::exp(-arg1);
-      BoysFunctionList_std[Nmax] = BoysFunction(Nmax, arg1);
+      BoysFunctionList_std[Nmax] = integralInfo.BoysFunction(Nmax, arg1);
       for(int i = Nmax-1; i >= 0; i--)
 	BoysFunctionList_std[i] = (2*arg1*BoysFunctionList_std[i+1] + expMinusArg1) / (2*i+1);
       /* Prepare BoysFunctionList_mod */
       /* Use downward recursion to get Boys function values */
       ergo_real arg2 =  alpha0 * R_12_squared * v1_squared;
       ergo_real expMinusArg2 = std::exp(-arg2);
-      BoysFunctionList_mod[Nmax] = BoysFunction(Nmax, arg2);
+      BoysFunctionList_mod[Nmax] = integralInfo.BoysFunction(Nmax, arg2);
       for(int i = Nmax-1; i >= 0; i--)
 	BoysFunctionList_mod[i] = (2*arg2*BoysFunctionList_mod[i+1] + expMinusArg2) / (2*i+1);
       // rescale
@@ -88,7 +88,7 @@ get_related_integrals_hermite(const monomial_info_struct* monomial_info,
       /* Compute all Boys function values needed */
       /* Use downward recursion to get Boys function values */
       ergo_real arg = alpha0 * R_12_squared;
-      BoysList[Nmax] = BoysFunction(Nmax, arg);
+      BoysList[Nmax] = integralInfo.BoysFunction(Nmax, arg);
       if(Nmax > 0)
 	{
 	  ergo_real expMinusArg = std::exp(-arg); 
@@ -690,13 +690,13 @@ get_related_integrals_hermite(const monomial_info_struct* monomial_info,
   for(n = Nmax - 1; n >= 0; n--)
     {
       int nn = Nmax - n;
-      int noOfMonomials = monomial_info->no_of_monomials_list[nn];
+      int noOfMonomials = integralInfo.monomial_info.no_of_monomials_list[nn];
       int i;
       for(i = 1; i < noOfMonomials; i++)
 	{
-	  int ix = monomial_info->monomial_list[i].ix;
-	  int iy = monomial_info->monomial_list[i].iy;
-	  int iz = monomial_info->monomial_list[i].iz;
+	  int ix = integralInfo.monomial_info.monomial_list[i].ix;
+	  int iy = integralInfo.monomial_info.monomial_list[i].iy;
+	  int iz = integralInfo.monomial_info.monomial_list[i].iz;
 	  if(ix > 0)
 	    {
 	      ergo_real Rval = dx0 * R[n+1][ix-1][iy][iz];
@@ -724,16 +724,16 @@ get_related_integrals_hermite(const monomial_info_struct* monomial_info,
   int i1, i2;
   for(i1 = 0; i1 < noOfMonomials_1; i1++)
     {
-      int ix1 = monomial_info->monomial_list[i1].ix;
-      int iy1 = monomial_info->monomial_list[i1].iy;
-      int iz1 = monomial_info->monomial_list[i1].iz;
+      int ix1 = integralInfo.monomial_info.monomial_list[i1].ix;
+      int iy1 = integralInfo.monomial_info.monomial_list[i1].iy;
+      int iz1 = integralInfo.monomial_info.monomial_list[i1].iz;
       int n1 = ix1+iy1+iz1;
       ergo_real prefactor = minus1topowList[n1] * resultPreFactor;
       for(i2 = 0; i2 < noOfMonomials_2; i2++)
 	{
-	  int ix2 = monomial_info->monomial_list[i2].ix;
-	  int iy2 = monomial_info->monomial_list[i2].iy;
-	  int iz2 = monomial_info->monomial_list[i2].iz;
+	  int ix2 = integralInfo.monomial_info.monomial_list[i2].ix;
+	  int iy2 = integralInfo.monomial_info.monomial_list[i2].iy;
+	  int iz2 = integralInfo.monomial_info.monomial_list[i2].iz;
 	  primitiveIntegralList[i1*noOfMonomials_2+i2] = prefactor * R[0][ix1+ix2][iy1+iy2][iz1+iz2];
 	} // END FOR i2  
     }
