@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,7 +52,7 @@ do_sort_int_list(int* list, int n)
 
 
 int
-organize_distributions(const IntegralInfo* integralInfo,
+organize_distributions(const IntegralInfo & integralInfo,
 		       DistributionSpecStructLabeled* distrList_in, 
 		       int distrCount, 
 		       distr_org_struct* result,
@@ -673,7 +673,7 @@ organize_distributions(const IntegralInfo* integralInfo,
   for(int i = 0; i < distrCount; i++)
     {
       minimalDistrList[i].coeff = distrList[i].distr.coeff;
-      minimalDistrList[i].monomialIndex = integralInfo->monomial_info.monomial_index_list
+      minimalDistrList[i].monomialIndex = integralInfo.monomial_info.monomial_index_list
 	[(int)distrList[i].distr.monomialInts[0]]
 	[(int)distrList[i].distr.monomialInts[1]]
 	[(int)distrList[i].distr.monomialInts[2]];
