@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -63,20 +63,6 @@ static Treal maxdiff_tri(const Treal* f1,const Treal* f2,int size);
 template<class Treal>
 static Treal frobdiff(const Treal* f1,const Treal* f2,int size);
 
-template<class T>
-static void fileread(T *ptr,int size,FILE*);
-template<>
-void fileread<double>(double *ptr,int size,FILE* file);
-template<>
-void fileread<float>(float *ptr,int size,FILE* file);
-
-template<class T>
-void filewrite(T *ptr,int size,FILE*);
-template<>
-void filewrite<double>(double *ptr,int size,FILE* file);
-template<>
-void filewrite<float>(float *ptr,int size,FILE* file);
-
 using namespace mat;
 
 template<typename Treal>
@@ -182,9 +168,9 @@ int mainFun(int argc,char* argv[]) {
     int size = 87; /* Use weird size to find more bugs. */
     int nlevels = 3;
     std::vector<int> blockSizes(nlevels);
-    blockSizes[nlevels - 1] = 1;
+    blockSizes[nlevels - 1] = 1; // should always be one
 #if 1
-    blockSizes[nlevels - 2] = 1;
+    blockSizes[nlevels - 2] = 1; // lowest level blocksize
     blockSizes[nlevels - 3] = 5;
 #else
     for (int ind = nlevels - 2; ind >= 0; ind--)
@@ -728,10 +714,10 @@ int mainFun(int argc,char* argv[]) {
       normalMatrix A, B, C;
       A.resetSizesAndBlocks(rows,cols);
       A.randomZeroStructure(0.7);
-      //      A.random();
+      //A.random();
       B.resetSizesAndBlocks(rows,cols);
       B.randomZeroStructure(0.7);
-      //      B.random();
+      //B.random();
       std::vector<real> valuesA;
       A.get_values(rowindex, colindex, valuesA);
       std::vector<real> valuesB;
@@ -1977,8 +1963,8 @@ int mainFun(int argc,char* argv[]) {
 	blockSizesTmp[nlevelsTmp - 1] = 1;
 	blockSizesTmp[nlevelsTmp - 2] = 1;
 	blockSizesTmp[nlevelsTmp - 3] = 5;
-	SizesAndBlocks rowsTmp(blockSizes, sizeTmp);
-	SizesAndBlocks colsTmp(blockSizes, sizeTmp);
+	SizesAndBlocks rowsTmp(blockSizesTmp, sizeTmp);
+	SizesAndBlocks colsTmp(blockSizesTmp, sizeTmp);
 	syA.resetSizesAndBlocks(rowsTmp,colsTmp);
 	// The 5 by 5 test matrix is zero except three values on the
 	// diagonal. This particular matrix appeared as the X-X2 matrix
@@ -2115,7 +2101,7 @@ int mainFun(int argc,char* argv[]) {
 	       <<thr
 	       <<"\n Received Error:     "
 	       <<std::setprecision(10)<<std::setw(15)
-	       <<syB.eucl(1e-11);
+	       <<syB.eucl(thr * 1e-7);
       
       if (syB.eucl(thr * 1e-3) < thr)
 	std::cout<<"   OK" <<std::endl;
@@ -2154,7 +2140,7 @@ int mainFun(int argc,char* argv[]) {
 		 <<thr
 		 <<"\n Received Error (Euclidean norm):     "
 		 <<std::setprecision(10)<<std::setw(15)
-		 <<syB.eucl(1e-11);
+		 <<syB.eucl(thr * 1e-7);
       
 	if (syB.eucl(thr * 1e-3) < thr)
 	  std::cout<<"   OK" <<std::endl;
@@ -2274,7 +2260,72 @@ int mainFun(int argc,char* argv[]) {
 	std::cout<<"   ERROR" <<std::endl;
 	std::exit(1);
       }
+
     } /* End of truncation using triple matrix as measure */
+
+    /* *********  Test truncation using congr trans measure again
+     *
+     * Truncation: Zt = Z + E
+     * Measure:     ||E^T * B * E + E^T * B * A + A^T * B * E||_2 < threshold
+     *
+     */ {
+
+      /********** Initialization of SizesAndBlocks                       */
+      int size = 87; /* Use weird size to find more bugs. */
+      int nlevels = 3;
+      std::vector<int> blockSizes(nlevels);
+      blockSizes[nlevels - 1] = 1;
+      blockSizes[nlevels - 2] = 1;
+      blockSizes[nlevels - 3] = 32;
+
+      std::cout << "Running tests with blocksize vector: ";
+      for (int ind = 0; ind < nlevels; ind++)
+	std::cout << blockSizes[ind] << "  ";
+      std::cout << std::endl;
+      
+      SizesAndBlocks rows2(blockSizes, size);
+      SizesAndBlocks cols2(blockSizes, size);
+      // Now we have rows2 and cols2, now we do the test again
+      real ONE = 1.0;
+      symmMatrix syA;
+      syA.resetSizesAndBlocks(rows2,cols2);
+      syA.randomZeroStructure(0.7);
+      symmMatrix syARef(syA);
+      triangMatrix trZ;
+      trZ.resetSizesAndBlocks(rows2,cols2);
+      expRule<real> er;
+      trZ.setElementsByRule(er);      
+      triangMatrix trZCopy(trZ);
+
+      std::cout<<"\nTest truncation with triple matrix measure again: \n";
+      syARef = transpose(trZ) * syARef * trZ;
+      real thr = 0.01;
+      real reported_error = trZ.eucl_thresh_congr_trans_measure(thr, syA);
+      syA = transpose(trZ) * syA * trZ;
+      syA += -ONE * syARef;
+      real err = syA.eucl(1e-7);
+      
+      std::cout<<"\n Requested threshold:"
+	       <<std::setprecision(10)<<std::setw(15)
+	       <<thr
+	       <<"\n Received Error:     "
+	       <<std::setprecision(10)<<std::setw(15)
+	       <<err
+	       <<"\n Reported Error:     "
+	       <<std::setprecision(10)<<std::setw(15)
+	       <<reported_error;
+      
+      if (err < thr)
+	std::cout<<"   OK" <<std::endl;
+      else {
+	std::cout<<"   ERROR" <<std::endl;
+	std::exit(1);
+      }
+
+      
+
+    } /* End of truncation using triple matrix as measure second time */
+
 #endif
 
 
@@ -2344,7 +2395,7 @@ int mainFun(int argc,char* argv[]) {
 		<< FileWritable::getStatsCountCopyAndAssign() << std::endl;
 
     }
-
+    
   }
   catch (Failure & e) {
     std::cout << "Failure caught: "<<e.what() << std::endl;
@@ -2418,40 +2469,3 @@ static Treal frobdiff(const Treal* f1,const Treal* f2,int size) {
   return template_blas_sqrt(diff);
 }
 
-template<class T>
-static void fileread(T *ptr,int size,FILE*) {
-  std::cout<<"error reading file"<<std::endl;
-}
-template<>
-void fileread<double>(double *ptr,int size,FILE* file) {
-  fread(ptr,sizeof(double),size*size,file);
-}
-template<>
-void fileread<float>(float *ptr,int size,FILE* file) {
-  double* tmpptr=new double [size*size];
-  fread(tmpptr,sizeof(double),size*size,file);
-  for (int i=0;i<size*size;i++)
-    {
-      ptr[i]=(float)tmpptr[i];
-    }
-  delete[] tmpptr;
-}
-
-template<class T>
-static void filewrite(T *ptr,int size,FILE*) {
-  std::cout<<"error writing file"<<std::endl;
-}
-template<>
-void filewrite<double>(double *ptr,int size,FILE* file) {
-  fwrite(ptr,sizeof(double),size*size,file);
-}
-template<>
-void filewrite<float>(float *ptr,int size,FILE* file) {
-  double* tmpptr=new double [size*size];
-  for (int i=0;i<size*size;i++) {
-    tmpptr[i]=(double)ptr[i];
-  }
-  fwrite(tmpptr,sizeof(double),size*size,file);
-  delete[] tmpptr;
-}
-
