@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@
 #include <math.h>
 #include <iomanip>
 #include "PuriStepInfo.h"
-#define __ID__ "$Id: PuriInfo.h 4437 2012-07-05 09:01:18Z elias $"
+#define __ID__ "$Id$"
 namespace mat {
   /** Contains information about a purification process
    *
@@ -116,6 +116,18 @@ namespace mat {
       inline Interval<Treal> getLumoF() const {return lumoF;}
       inline int getMaxSteps() const {return maxSteps;}
       inline int getNSteps() const {return nSteps;}
+      /* Returns a vector or 0 and 1 giving the used poly sequence. */
+      void getPolys(std::vector<int> & resultVector) {
+	resultVector.resize(nSteps);
+	for(int i = 0; i < nSteps; i++)
+	  resultVector[i] = step[i].getPoly();
+      }
+      /* Returns a vector of chosen threshold values. */
+      void getThreshValues(std::vector<Treal> & resultVector) {
+	resultVector.resize(nSteps);
+	for(int i = 0; i < nSteps; i++)
+	  resultVector[i] = step[i].getChosenThresh();
+      }
       /* Tries to improve the homoF and lumoF eigenvalues.
        * Called after the purification process.
        */
