@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@
 #include <math.h>
 #include <iomanip>
 #include "PuriStepInfoDebug.h"
-#define __ID__ "$Id: PuriStepInfo.h 4437 2012-07-05 09:01:18Z elias $"
+#define __ID__ "$Id$"
 namespace mat {
   /* IDEA: Adjust threshold (looser) in later iteration if threshold in early
    * iteration was tighter than expected.
@@ -108,7 +108,7 @@ namespace mat {
 	delete eigVecPtr;
 	eigVecPtr = eigVecPtr_;
       }
-      Tvector const * const getEigVecPtr() const {return eigVecPtr;}
+      Tvector const * getEigVecPtr() const {return eigVecPtr;}
 
       void improveHomoLumo(Interval<Treal> const homoInt, 
 			   Interval<Treal> const lumoInt);
