@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,7 +59,7 @@ namespace mat{
     void allocate() {
       assert(!this->is_empty());
       assert(this->is_zero());
-      this->elements = new Telement[this->n()];
+      this->elements = allocateElements<Telement>(this->n());
       SizesAndBlocks rowSAB;
       for (int row = 0; row < this->rows.getNBlocks(); row++) {
 	rowSAB = this->rows.getSizesAndBlocksForLowerLevel(row);
@@ -185,7 +185,7 @@ namespace mat{
 
   template<class Treal, class Telement>
     void Vector<Treal, Telement>::clear() {
-    delete[] this->elements;
+    freeElements(this->elements);
     this->elements = 0;
   }
 
@@ -503,7 +503,7 @@ namespace mat{
     void allocate() {
       assert(!this->is_empty());
       assert(this->is_zero());
-      this->elements = new Treal[this->n()];
+      this->elements = allocateElements<Treal>(this->n());
       for (int ind = 0; ind < this->n(); ind++) 
 	this->elements[ind] = 0;
     }
@@ -616,7 +616,7 @@ namespace mat{
 
   template<class Treal>
     void Vector<Treal>::clear() {
-    delete[] this->elements;
+    freeElements(this->elements);
     this->elements = 0;
   }
 
