@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -73,7 +73,7 @@ namespace mat{
     inline bool is_zero() const {return !elements;}
     
     inline void resetRows(SizesAndBlocks const & newRows) {
-      delete[] elements;
+      freeElements(elements);
       elements = 0;
       rows = newRows;
     }
@@ -116,9 +116,9 @@ namespace mat{
 #if 0
     inline void assert_alloc() {
       if (this->cap < this->nel) {
-	delete[] this->elements;
+	freeElements(this->elements);
 	this->cap = this->nel;      
-	this->elements = new Telement[this->cap];
+	this->elements = allocateElements<Telement>(this->cap);
 	for (int ind = 0; ind < this->cap; ind++)
 	  this->elements[ind] = 0;
       }
@@ -137,7 +137,7 @@ namespace mat{
     (const VectorHierarchicBase<Treal, Telement>& vec) 
     : rows(vec.rows) {
       if (!vec.is_zero()) {
-	elements = new Telement[n()];
+	elements = allocateElements<Telement>(n());
 	for (int i = 0; i < n(); i++) 
 	  elements[i] = vec.elements[i];
       }
@@ -150,13 +150,13 @@ namespace mat{
       operator=(const VectorHierarchicBase<Treal, Telement>& vec) {
       if (vec.is_zero()) { /* Condition also matches empty matrices. */
 	rows = vec.rows;
-	delete[] elements;
+	freeElements(elements);
 	elements = 0;
 	return *this;
       }
       if (is_zero() || (n() != vec.n())) {
-	delete[] elements;
-	elements = new Telement[vec.n()];
+	freeElements(elements);
+	elements = allocateElements<Telement>(vec.n());
       }
       rows = vec.rows;
       for (int i = 0; i < n(); i++)
@@ -168,7 +168,7 @@ namespace mat{
     template<class Treal, class Telement>
       VectorHierarchicBase<Treal, Telement>::
       ~VectorHierarchicBase() {
-      delete[] elements;
+      freeElements(elements);
     }
     
 } /* end namespace mat */
