@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,19 +61,9 @@ int mainFun(int size, int parallel_level, int block_size,
   typedef Matrix<real, Mat_1> Mat_2;
   typedef Matrix<real, Mat_2> Mat_3;
   typedef Matrix<real, Mat_3> Mat_4;
-  typedef Vector<real, real > Vec_1;
-  typedef Vector<real, Vec_1> Vec_2;
-  typedef Vector<real, Vec_2> Vec_3;
-  typedef Vector<real, Vec_3> Vec_4;
   
   typedef Mat_4 matri;
-  typedef Vec_4 vect;
-  typedef MatrixSymmetric<real, matri> symmMatrix;
-  typedef MatrixTriangular<real, matri> triangMatrix;
   typedef MatrixGeneral<real, matri> normalMatrix;
-  typedef VectorGeneral<real, vect> normalVector;
-  typedef arn::LanczosLargestMagnitudeEig<real, symmMatrix, normalVector>
-    myLanczosType;
   
   try {
 
