@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,11 +45,16 @@
 #include <ctime>
 #include "Failure.h"
 
-// We need to include config.h to get USE_LINALG_TEMPLATES flag
+/* We need to include config.h to get USE_LINALG_TEMPLATES and USE_SSE_INTRINSICS flags. */
 #include "config.h"
 
 #include "template_lapack_common.h"
 
+#ifdef USE_SSE_INTRINSICS
+#include "Memory_buffer_thread.h"
+#include "gemm_sse/gemm_sse.h"
+#endif
+
 /* LEVEL 3 */
 extern "C" void dgemm_(const char *ta,const char *tb,
 		       const int *n, const int *k, const int *l,
@@ -227,7 +232,26 @@ namespace mat
 			    const T *alpha,const T *A,const int *lda,
 			    const T *B, const int *ldb,
 			    const T *beta,T *C, const int *ldc) {
-    template_blas_gemm(ta,tb,n,k,l,alpha,A,lda,B,ldb,beta,C,ldc);
+#ifdef USE_SSE_INTRINSICS
+    if (*ta == 'N' && *tb == 'N' && *n == 32 && *k == 32 && *l == 32 && *alpha == 1.0 && *beta == 1) {
+      static T * A_packed;
+      static T * B_packed;
+      static T * C_packed;
+      int pack_max_size = 10000;  
+      if ( (pack_max_size*sizeof(T))%16 )
+	throw std::runtime_error("In gblas gemm: requested buffer size not multiple of 16 bytes");
+      static T * buffer;
+      Memory_buffer_thread::instance().get_buffer(pack_max_size*3, buffer);
+      A_packed = buffer;
+      B_packed = buffer + pack_max_size;
+      C_packed = buffer + 2*pack_max_size;
+      gemm_sse<T>(A,B,C,32,32,32,
+		  A_packed, B_packed, C_packed, 
+		  pack_max_size, pack_max_size, pack_max_size);  
+    }
+    else
+#endif
+      template_blas_gemm(ta,tb,n,k,l,alpha,A,lda,B,ldb,beta,C,ldc);
   }
 
   
