@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,10 @@
 #define TEMPLATE_LAPACK_LARRE_HEADER
 
 
+#include "template_lapack_larrk.h"
+#include "template_lapack_lasq2.h"
+
+
 template<class Treal>
 int template_lapack_larre(const char *range, const integer *n, Treal *vl, 
 	Treal *vu, integer *il, integer *iu, Treal *d__, Treal 
