@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,10 @@
 #define TEMPLATE_LAPACK_TGEVC_HEADER
 
 
+#include "template_lapack_labad.h"
+#include "template_lapack_lacpy.h"
+
+
 template<class Treal>
 int template_lapack_tgevc(const char *side, const char *howmny, const logical *select, 
 	const integer *n, const Treal *a, const integer *lda, const Treal *b, const integer *ldb, 
