@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -353,7 +353,8 @@ namespace mat { /* Matrix namespace */
 											Treal const threshold ) {
     Treal Afrob = this->A.frob();
     Treal Bfrob = B.frob();
-    lowTrunc    = -Afrob + std::sqrt( Afrob*Afrob + threshold / Bfrob );
+    Treal tmp = -Afrob + std::sqrt( Afrob*Afrob + threshold / Bfrob );
+    lowTrunc = tmp*tmp; 
     highTrunc   = std::numeric_limits<Treal>::max();
   }
   
