@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -134,6 +134,8 @@ SCF_general::SCF_general(const Molecule& molecule_,
   if ( scfopts.create_basis_func_coord_file == 1 ) {
     write_basis_func_coord_file(basisInfo);
   }
+  if ( scfopts.create_2el_integral_m_file == 1 ) 
+    write_2el_integral_m_file(basisInfo, integralInfo);
 
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Computing Euclidean norm of overlap matrix (just for fun)...");  
   Util::TimeMeter timeMeterEuclS;
@@ -227,7 +229,7 @@ SCF_general::SCF_general(const Molecule& molecule_,
   else
     {
       if(scfopts.use_simple_dense_H_core == 1) {
-	if(extraCharges.noOfAtoms != 0) {
+	if(extraCharges.getNoOfAtoms() != 0) {
 	  do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error: (extraCharges.noOfAtoms != 0) not implemented for use_simple_dense_H_core case.");
 	  throw "error: (extraCharges.noOfAtoms != 0) not implemented for use_simple_dense_H_core case.";
 	}
@@ -288,6 +290,15 @@ SCF_general::SCF_general(const Molecule& molecule_,
   output_sparsity_symm(n, H_core_Matrix, "H_core_Matrix after trunc");
   
   output_current_memory_usage(LOG_AREA_SCF, "after getting H_core_Matrix");
+
+  if ( scfopts.create_mtx_file_H_core == 1 ) {
+    // Write H_core matrix in matrix market format
+    std::stringstream ss_id;
+    ss_id << scfopts.calculation_identifier << " - H_core matrix";
+    write_matrix_in_matrix_market_format( H_core_Matrix, matOpts.inversePermutationHML, "H_core_matrix", 
+					  ss_id.str(), scfopts.method_and_basis_set );
+  }
+
   H_core_Matrix.writeToFile();
   output_current_memory_usage(LOG_AREA_SCF, "after writing H_core_Matrix to file");
   
@@ -333,6 +344,14 @@ void SCF_general::get_overlap_matrix(symmMatrix & S)
 }
 
 
+void SCF_general::get_invCholFactor_matrix(triangMatrix & invCholFactor_)
+{
+  invCholFactor.readFromFile();
+  invCholFactor_ = invCholFactor;
+  invCholFactor.writeToFile();
+}
+
+
 void SCF_general::get_H_core_matrix(symmMatrix & H_core)
 {
   H_core_Matrix.readFromFile();
@@ -658,6 +677,11 @@ void SCF_general::do_SCF_iterations()
       get_new_density_matrix();
       timeMeterGetNewDensityMatrix.print(LOG_AREA_SCF, "get_new_density_matrix");
       curr_cycle_stats->stop_timer("get_new_density_matrix");
+      
+      // At this point a new density matrix has just been computed, so the electronic entropy term has also been computed in the nonzero-temperature case.
+      if(scfopts.electronic_temperature > 0)
+	do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Electronic entropy term: %22.11f, energy with entropy term included: %22.11f", 
+		  (double)electronicEntropyTerm, (double)(energy + electronicEntropyTerm));
 
       if(step > 1)
 	report_density_difference();
@@ -701,6 +725,7 @@ void SCF_general::do_SCF_iterations()
       if(scfopts.output_statistics_mfiles)
 	curr_cycle_stats->output_mfile( ss.str() );
       delete curr_cycle_stats;
+      curr_cycle_stats = NULL; // since curr_cycle_stats is deleted in destructor, we need to set to null here to avoid double-delete if we exit loop in some unusual way.
     } // END WHILE main SCF loop
 
   timeMeterScfMainLoop.print(LOG_AREA_SCF, "Main SCF loop");
@@ -719,6 +744,9 @@ void SCF_general::do_SCF_iterations()
   if(scfopts.output_mulliken_pop == 1)
     do_mulliken_pop_stuff();
 
+  if(scfopts.compute_gradient_fixeddens == 1)
+    compute_gradient_fixeddens();
+
   if(scfopts.save_full_matrices_for_matlab == 1)
     save_full_matrices_for_matlab();
   
@@ -734,9 +762,6 @@ void SCF_general::do_SCF_iterations()
   if(scfopts.output_csr_matrices_for_gao == 1)
     output_csr_matrices_for_gao();
 
-  if(scfopts.do_electron_dynamics == 1)
-    do_electron_dynamics();
-
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, "SCF_general::do_SCF_iterations finished.");
   timeMeterTot.print(LOG_AREA_SCF, "SCF_general::do_SCF_iterations");
 }
