@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,6 +46,7 @@ class SCF_general
   void do_SCF_iterations();
 
   void get_overlap_matrix(symmMatrix & S);
+  void get_invCholFactor_matrix(triangMatrix & invCholFactor_);
   void get_H_core_matrix(symmMatrix & H_core);
   void get_energy(ergo_real & E, ergo_real & E_nuclear);
 
@@ -89,6 +90,8 @@ class SCF_general
   ergo_real energy_2el;
   ergo_real energy;
 
+  ergo_real electronicEntropyTerm;
+
   ergo_real errorMeasure;
 
   ergo_real curr_subspace_diff;
@@ -129,7 +132,6 @@ class SCF_general
   virtual void add_random_disturbance_to_starting_guess() = 0;
   virtual void output_density_images() = 0;
   virtual void output_csr_matrices_for_gao() = 0;
-  virtual void do_electron_dynamics() = 0;
   virtual void write_diag_dens_to_file() = 0;
   virtual void report_final_results() = 0;
   virtual void save_density_as_prevdens() = 0;
@@ -147,6 +149,7 @@ class SCF_general
   virtual void create_gabedit_file() const = 0;
   virtual void compute_dipole_moment() = 0;
   virtual void do_mulliken_pop_stuff() = 0;
+  virtual void compute_gradient_fixeddens() = 0;
 };
 
 
