@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,7 +37,6 @@
 #include "matrix_utilities.h"
 #include "matrix_utilities_gao.h"
 #include "machine_epsilon.h"
-#include "tdhf_dynamics.h"
 #include "units.h"
 #include "atom_labels.h"
 
@@ -75,6 +74,22 @@ SCF_restricted::~SCF_restricted()
 }
 
 
+void SCF_restricted::get_Fock_matrix(symmMatrix & FockMatrix_)
+{
+  FockMatrix.readFromFile();
+  FockMatrix_ = FockMatrix;
+  FockMatrix.writeToFile();
+}
+
+
+void SCF_restricted::get_density_matrix(symmMatrix & densityMatrix_)
+{
+  densityMatrix.readFromFile();
+  densityMatrix_ = densityMatrix;
+  densityMatrix.writeToFile();
+}
+
+
 void SCF_restricted::initialize_matrices()
 {
   densityMatrix.resetSizesAndBlocks(matOpts.size_block_info,
@@ -179,6 +194,7 @@ void SCF_restricted::get_starting_guess_density()
 	  intervalType lumoInterval_dummy2(-1e22,1e22);
 	  densityMatrix.writeToFile();
 	  std::map<std::string, double> puri_stats;
+	  ergo_real entropyTermDummy = 0;
 	  if(get_dens_from_fock_general(n,
 					noOfElectrons / 2,
 					scfopts.use_diagonalization,
@@ -186,6 +202,7 @@ void SCF_restricted::get_starting_guess_density()
 					scfopts.electronic_temperature,
 					densityMatrix,
 					2,
+					entropyTermDummy,
 					H_core_Matrix,
 					homoInterval_dummy1,
 					lumoInterval_dummy1,
@@ -208,7 +225,8 @@ void SCF_restricted::get_starting_guess_density()
 					puri_stats,
 					scfopts.do_sparsity_investigation,
 					scfopts.sparsity_plots_resolution_m,
-					scfopts.do_comparison_to_simple_purification) != 0)
+					scfopts.do_comparison_to_simple_purification,
+					scfopts.do_puri_mmul_tests) != 0)
 	    {
 	      throw "SCF_restricted::get_starting_guess_density: Error in get_dens_from_fock_general";
 	    }
@@ -355,7 +373,11 @@ void SCF_restricted::get_2e_part_and_energy()
       throw "error in get_2e_matrix_and_energy_sparse";
     }
   densityMatrix.writeToFile();
-	
+
+  // Check that G matrix is free from "inf", "nan" etc.
+  if(check_if_matrix_contains_strange_elements(G, matOpts.inversePermutationHML))
+    throw std::runtime_error("error in SCF_restricted::get_2e_part_and_energy(): G matrix contains inf or nan.");
+  
   // calculate Fock matrix F = H_core + G
   H_core_Matrix.readFromFile();
   FockMatrix.readFromFile();
@@ -714,6 +736,7 @@ void SCF_restricted::get_new_density_matrix()
 				    scfopts.electronic_temperature,
 				    densityMatrix,
 				    2,
+				    electronicEntropyTerm,
 				    FockMatrix, // should be same as Fprev now
 				    homoInterval_Fprev, // output
 				    lumoInterval_Fprev, // output
@@ -736,7 +759,8 @@ void SCF_restricted::get_new_density_matrix()
 				    puri_stats,
 				    scfopts.do_sparsity_investigation,
 				    scfopts.sparsity_plots_resolution_m,
-				    0) != 0)
+				    0,
+				    scfopts.do_puri_mmul_tests) != 0)
 	{
 	  throw "SCF_restricted::get_new_density_matrix: Error in get_dens_from_fock_general";
 	}
@@ -783,6 +807,7 @@ void SCF_restricted::get_new_density_matrix()
 				scfopts.electronic_temperature,
 				densityMatrix,
 				2,
+				electronicEntropyTerm,
 				*F_effective, // should normally be same as Fprev now
 				homoInterval_Fprev, // output
 				lumoInterval_Fprev, // output
@@ -806,6 +831,7 @@ void SCF_restricted::get_new_density_matrix()
 				scfopts.do_sparsity_investigation,
 				scfopts.sparsity_plots_resolution_m,
 				scfopts.do_comparison_to_simple_purification,
+				scfopts.do_puri_mmul_tests,
 				&eigVecLUMO,
 				&eigVecHOMO) != 0)
     {
@@ -889,10 +915,42 @@ void SCF_restricted::save_full_matrices_for_matlab()
 
 void SCF_restricted::output_density_images()
 {
-  do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error: SCF_restricted::output_density_images not implemented.");
-  throw "error: SCF_restricted::output_density_images not implemented.";
+  Util::TimeMeter timeMeter;
+
+  int n = basisInfo.noOfBasisFuncs;
+
+  ergo_real* densityMatrixFull_tot  = new ergo_real[n*n];
+  ergo_real* densityMatrixFull_spin = new ergo_real[n*n];
+
+  // Get full matrix version of density matrix, and empty spin density matrix.
+  {
+    std::vector<ergo_real> densityMatrixFull(n*n);
+    
+    densityMatrix.readFromFile();
+    densityMatrix.fullMatrix(densityMatrixFull,
+			     matOpts.inversePermutationHML,
+			     matOpts.inversePermutationHML);
+    densityMatrix.writeToFile();
+        
+    for(int i = 0; i < n*n; i++)
+      {
+	densityMatrixFull_tot [i] = densityMatrixFull[i];
+	densityMatrixFull_spin[i] = 0;
+      }
+  }
+
+  do_density_images(basisInfo,
+		    molecule,
+		    densityMatrixFull_tot, 
+		    densityMatrixFull_spin,
+		    scfopts.output_density_images_boxwidth);
+
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "SCF_restricted::output_density_images finished OK.");
+  timeMeter.print(LOG_AREA_SCF, "SCF_restricted::output_density_images");
 }
 
+
+
 void SCF_restricted::output_csr_matrices_for_gao()
 {
   int n = basisInfo.noOfBasisFuncs;
@@ -915,26 +973,6 @@ void SCF_restricted::output_csr_matrices_for_gao()
 }
 
 
-void SCF_restricted::do_electron_dynamics()
-{
-  FockMatrix.readFromFile();
-  densityMatrix.readFromFile();
-  S_symm.readFromFile();
-  invCholFactor.readFromFile();
-  do_tdhf_dynamics(basisInfo,
-		   integralInfo,
-		   molecule,
-		   extraCharges,
-		   matOpts,
-		   CAM_params,
-		   J_K_params,
-		   FockMatrix,
-		   densityMatrix,
-		   S_symm,
-		   invCholFactor);
-}
-
-
 void SCF_restricted::write_diag_dens_to_file()
 {
   int n = basisInfo.noOfBasisFuncs;
@@ -1158,27 +1196,27 @@ void SCF_restricted::create_gabedit_file() const
   int use_6_d_funcs = 0;
   ff << "[Gabedit Format] Sphe" << std::endl;
   ff << "[Atoms] Angs" << std::endl;
-  for(int i = 0; i < molecule.noOfAtoms; i++) {
+  for(int i = 0; i < molecule.getNoOfAtoms(); i++) {
     char atomLabelString[4];
-    get_atom_label_from_charge_int(molecule.atoms[i].charge, atomLabelString, 4);
-    ff << atomLabelString << " " << i+1 << " " << molecule.atoms[i].charge
-       << "   " << molecule.atoms[i].coords[0] / UNIT_one_Angstrom
-       << "   " << molecule.atoms[i].coords[1] / UNIT_one_Angstrom
-       << "   " << molecule.atoms[i].coords[2] / UNIT_one_Angstrom 
+    get_atom_label_from_charge_int(molecule.getAtom(i).charge, atomLabelString, 4);
+    ff << atomLabelString << " " << i+1 << " " << molecule.getAtom(i).charge
+       << "   " << molecule.getAtom(i).coords[0] / UNIT_one_Angstrom
+       << "   " << molecule.getAtom(i).coords[1] / UNIT_one_Angstrom
+       << "   " << molecule.getAtom(i).coords[2] / UNIT_one_Angstrom 
        << std::endl;
   }
   std::vector<int> shellIdxList(basisInfo.noOfShells);
   int shellIdxCounter = 0;
   SquareFuncIntegrator sfi;
   ff << "[Basis]" << std::endl;
-  for(int i = 0; i < molecule.noOfAtoms; i++) {
+  for(int i = 0; i < molecule.getNoOfAtoms(); i++) {
     ff << i+1 << " 0" << std::endl;
     // Now output info about shells for this atom.
     for(int k = 0; k < basisInfo.noOfShells; k++) {
       // Check if this shell belongs to the current atom.
-      ergo_real absdx = std::abs(basisInfo.shellList[k].centerCoords[0] - molecule.atoms[i].coords[0]);
-      ergo_real absdy = std::abs(basisInfo.shellList[k].centerCoords[1] - molecule.atoms[i].coords[1]);
-      ergo_real absdz = std::abs(basisInfo.shellList[k].centerCoords[2] - molecule.atoms[i].coords[2]);
+      ergo_real absdx = std::abs(basisInfo.shellList[k].centerCoords[0] - molecule.getAtom(i).coords[0]);
+      ergo_real absdy = std::abs(basisInfo.shellList[k].centerCoords[1] - molecule.getAtom(i).coords[1]);
+      ergo_real absdz = std::abs(basisInfo.shellList[k].centerCoords[2] - molecule.getAtom(i).coords[2]);
       ergo_real distlimit = 0.01;
       if(absdx > distlimit || absdy > distlimit || absdz > distlimit)
 	continue;
@@ -1375,6 +1413,7 @@ void SCF_restricted::disturb_dens_matrix_exact_try(const symmMatrix & randomMatr
   int use_diag_on_error = 0;
   int n = basisInfo.noOfBasisFuncs;
   std::map<std::string, double> puri_stats;
+  ergo_real electronicEntropyTermDummy = 0;
   if(get_dens_from_fock_general(n,
 				noOfElectrons / 2,
 				use_diag,
@@ -1382,6 +1421,7 @@ void SCF_restricted::disturb_dens_matrix_exact_try(const symmMatrix & randomMatr
 				scfopts.electronic_temperature,
 				resultDensMatrix,
 				2,
+				electronicEntropyTermDummy,
 				SDS_symm,
 				homoInterval_tmp1,
 				lumoInterval_tmp1,
@@ -1404,7 +1444,8 @@ void SCF_restricted::disturb_dens_matrix_exact_try(const symmMatrix & randomMatr
 				puri_stats,
 				scfopts.do_sparsity_investigation,
 				scfopts.sparsity_plots_resolution_m,
-				scfopts.do_comparison_to_simple_purification) != 0)
+				scfopts.do_comparison_to_simple_purification,
+				scfopts.do_puri_mmul_tests) != 0)
     {
       throw "SCF_restricted::get_starting_guess_density: Error in get_dens_from_fock_general";
     }
@@ -1503,6 +1544,7 @@ void SCF_restricted::disturb_fock_matrix(ergo_real subspaceError)
 				scfopts.electronic_temperature,
 				densityMatrix_dummy,
 				2,
+				electronicEntropyTerm,
 				FockMatrix,
 				homoInterval_tmp1,
 				lumoInterval_tmp1,
@@ -1525,7 +1567,8 @@ void SCF_restricted::disturb_fock_matrix(ergo_real subspaceError)
 				puri_stats,
 				scfopts.do_sparsity_investigation,
 				scfopts.sparsity_plots_resolution_m,
-				scfopts.do_comparison_to_simple_purification) != 0)
+				scfopts.do_comparison_to_simple_purification,
+				scfopts.do_puri_mmul_tests) != 0)
     {
       throw "SCF_restricted::get_starting_guess_density: Error in get_dens_from_fock_general";
     }
@@ -1553,4 +1596,76 @@ void SCF_restricted::disturb_fock_matrix(ergo_real subspaceError)
 }
 
 
+static ergo_real get_nucl_energy_for_given_mol_and_dens(const IntegralInfo& integralInfo,
+							const Molecule& molecule,
+							const BasisInfoStruct& basisInfo,
+							const symmMatrix & D,
+							ergo_real threshold_integrals_1el,
+							mat::SizesAndBlocks const & matrix_size_block_info,
+							std::vector<int> const & permutationHML) {
+  ergo_real nuclearRepulsionEnergy = molecule.getNuclearRepulsionEnergy();
+  ergo_real elecNuclEnergy = get_electron_nuclear_attraction_energy(integralInfo,
+								    molecule,
+								    basisInfo,
+								    D,
+								    threshold_integrals_1el,
+								    matrix_size_block_info,
+								    permutationHML);
+  return nuclearRepulsionEnergy + elecNuclEnergy;
+}
+
+
+/* Compute gradient of energy with respect to nuclear positions, for
+   fixed electron density. */
+void
+SCF_restricted::compute_gradient_fixeddens() {
+  // Since we here regard the electron density as fixed, there are
+  // only two terms in the energy which give nonzero contributions:
+  // the nuclear-electron interaction term, and the nuclear-nuclear
+  // interaction term.
+  densityMatrix.readFromFile();
+  int nAtoms = molecule.getNoOfAtoms();
+  std::vector<ergo_real> result(nAtoms*3);
+  for(int i = 0; i < nAtoms; i++) {
+    for(int coordIdx = 0; coordIdx < 3; coordIdx++) {
+      const ergo_real h = 1e-3;
+      Molecule moleculeTmp = molecule;
+      Atom atomTmp = molecule.getAtom(i);
+      atomTmp.coords[coordIdx] += h;
+      moleculeTmp.replaceAtom(i, atomTmp);
+      ergo_real E1 = get_nucl_energy_for_given_mol_and_dens(integralInfo,
+							    moleculeTmp,
+							    basisInfo,
+							    densityMatrix,
+							    threshold_integrals_1el,
+							    matOpts.size_block_info,
+							    matOpts.permutationHML);
+      moleculeTmp = molecule;
+      atomTmp = molecule.getAtom(i);
+      atomTmp.coords[coordIdx] -= h;
+      moleculeTmp.replaceAtom(i, atomTmp);
+      ergo_real E2 = get_nucl_energy_for_given_mol_and_dens(integralInfo,
+							    moleculeTmp,
+							    basisInfo,
+							    densityMatrix,
+							    threshold_integrals_1el,
+							    matOpts.size_block_info,
+							    matOpts.permutationHML);
+      ergo_real gradientComponent = (E1 - E2) / (2 * h);
+      result[i*3+coordIdx] = gradientComponent;
+    } // END FOR coordIdx
+  } // END FOR i
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Gradient of energy with respect to nuclear positions, for fixed electron density:");
+  for(int i = 0; i < nAtoms; i++) {
+    do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Atom %6d: %22.11f %22.11f %22.11f", 
+	      i,
+	      result[i*3+0],
+	      result[i*3+1],
+	      result[i*3+2]);
+  }
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "(End of gradient)");
+  densityMatrix.writeToFile();
+}
+
+
 
