@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,6 +51,9 @@ class SCF_restricted : public SCF_general
   // Destructor
   ~SCF_restricted();
 
+  void get_Fock_matrix(symmMatrix & FockMatrix_);
+  void get_density_matrix(symmMatrix & densityMatrix_);
+
  private:
   void initialize_matrices();
   void check_params();
@@ -75,7 +78,6 @@ class SCF_restricted : public SCF_general
   void add_random_disturbance_to_starting_guess();
   void output_density_images();
   void output_csr_matrices_for_gao();
-  void do_electron_dynamics();
   void write_diag_dens_to_file();
   void report_final_results();
   void save_density_as_prevdens();
@@ -93,6 +95,7 @@ class SCF_restricted : public SCF_general
   void create_gabedit_file() const;
   void compute_dipole_moment();
   void do_mulliken_pop_stuff();
+  void compute_gradient_fixeddens();
 
   void get_non_ort_err_mat_normalized_in_ort_basis(symmMatrix & randomMatrix, int transform_with_S_also);
   void transform_with_S(symmMatrix & A);
