@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@
 #include "density_description_file_2.h"
 #include "densitymanager.h"
 #include "matrix_utilities.h"
+#include "units.h"
 
 
 SCF_unrestricted::SCF_unrestricted(const Molecule& molecule_,
@@ -229,6 +230,7 @@ void SCF_unrestricted::get_starting_guess_density()
 	  intervalType lumoInterval_dummy2_alpha(-1e22,1e22);
 	  densityMatrix_alpha.writeToFile();
 	  std::map<std::string, double> puri_stats_alpha;
+	  ergo_real electronicEntropyTermDummy = 0;
 	  if(get_dens_from_fock_general(n,
 					noOfElectrons_alpha,
 					scfopts.use_diagonalization,
@@ -236,6 +238,7 @@ void SCF_unrestricted::get_starting_guess_density()
 					scfopts.electronic_temperature,
 					densityMatrix_alpha,
 					1,
+					electronicEntropyTermDummy,
 					H_core_Matrix, 
 					homoInterval_dummy1_alpha,
 					lumoInterval_dummy1_alpha,
@@ -258,7 +261,8 @@ void SCF_unrestricted::get_starting_guess_density()
 					puri_stats_alpha,
 					scfopts.do_sparsity_investigation,
 					scfopts.sparsity_plots_resolution_m,
-					scfopts.do_comparison_to_simple_purification) != 0)
+					scfopts.do_comparison_to_simple_purification,
+					scfopts.do_puri_mmul_tests) != 0)
 	    {
 	      throw "SCF_unrestricted::get_starting_guess_density: Error in get_dens_from_fock_general for alpha.";
 	    }
@@ -281,6 +285,7 @@ void SCF_unrestricted::get_starting_guess_density()
 					scfopts.electronic_temperature,
 					densityMatrix_beta,
 					1,
+					electronicEntropyTermDummy,
 					H_core_Matrix, 
 					homoInterval_dummy1_beta,
 					lumoInterval_dummy1_beta,
@@ -303,7 +308,8 @@ void SCF_unrestricted::get_starting_guess_density()
 					puri_stats_beta,
 					scfopts.do_sparsity_investigation,
 					scfopts.sparsity_plots_resolution_m,
-					scfopts.do_comparison_to_simple_purification) != 0)
+					scfopts.do_comparison_to_simple_purification,
+					scfopts.do_puri_mmul_tests) != 0)
 	    {
 	      throw "SCF_unrestricted::get_starting_guess_density: Error in get_dens_from_fock_general for beta.";
 	    }
@@ -806,6 +812,7 @@ void SCF_unrestricted::get_new_density_matrix()
 
   // ALPHA
   std::map<std::string, double> puri_stats_alpha;
+  ergo_real electronicEntropyTerm_alpha = 0;
   if(get_dens_from_fock_general(n,
 				noOfElectrons_alpha, 
 				scfopts.use_diagonalization,
@@ -813,6 +820,7 @@ void SCF_unrestricted::get_new_density_matrix()
 				scfopts.electronic_temperature,
 				densityMatrix_alpha,
 				1,
+				electronicEntropyTerm_alpha,
 				FockMatrix_alpha, 
 				homoInterval_Fprev_alpha,
 				lumoInterval_Fprev_alpha,
@@ -835,13 +843,15 @@ void SCF_unrestricted::get_new_density_matrix()
 				puri_stats_alpha,
 				scfopts.do_sparsity_investigation,
 				scfopts.sparsity_plots_resolution_m,
-				scfopts.do_comparison_to_simple_purification) != 0)
+				scfopts.do_comparison_to_simple_purification,
+				scfopts.do_puri_mmul_tests) != 0)
     {
       throw "SCF_unrestricted::get_new_density_matrix: Error in get_dens_from_fock_general for alpha.";
     }
 
   // BETA
   std::map<std::string, double> puri_stats_beta;
+  ergo_real electronicEntropyTerm_beta = 0;
   if(get_dens_from_fock_general(n,
 				noOfElectrons_beta,
 				scfopts.use_diagonalization,
@@ -849,6 +859,7 @@ void SCF_unrestricted::get_new_density_matrix()
 				scfopts.electronic_temperature,
 				densityMatrix_beta,
 				1,
+				electronicEntropyTerm_beta,
 				FockMatrix_beta, 
 				homoInterval_Fprev_beta,
 				lumoInterval_Fprev_beta,
@@ -871,11 +882,13 @@ void SCF_unrestricted::get_new_density_matrix()
 				puri_stats_beta,
 				scfopts.do_sparsity_investigation,
 				scfopts.sparsity_plots_resolution_m,
-				scfopts.do_comparison_to_simple_purification) != 0)
+				scfopts.do_comparison_to_simple_purification,
+				scfopts.do_puri_mmul_tests) != 0)
     {
       throw "SCF_unrestricted::get_new_density_matrix: Error in get_dens_from_fock_general for beta.";
     }
 
+  electronicEntropyTerm = electronicEntropyTerm_alpha + electronicEntropyTerm_beta;
 
   // Report trace(DS) and spin info.
   S_symm.readFromFile();
@@ -995,160 +1008,12 @@ void SCF_unrestricted::output_density_images()
       }
   }
 
-  int nPrims1 = 0;
-  int nPrims2 = 0;
-  
-  DistributionSpecStruct* rho1 = NULL;
-  DistributionSpecStruct* rho2 = NULL;
-  ergo_real cutoff = 1e-8;
-  nPrims1 = get_no_of_primitives_for_density(cutoff, densityMatrixFull_tot , basisInfo);
-  nPrims2 = get_no_of_primitives_for_density(cutoff, densityMatrixFull_spin, basisInfo);
-  rho1 = new DistributionSpecStruct[nPrims1];
-  rho2 = new DistributionSpecStruct[nPrims2];
+  do_density_images(basisInfo,
+		    molecule,
+		    densityMatrixFull_tot, 
+		    densityMatrixFull_spin,
+		    scfopts.output_density_images_boxwidth);
 
-  int n1 = get_density(basisInfo,
-		       densityMatrixFull_tot,
-		       cutoff,
-		       nPrims1,
-		       rho1);      
-  int n2 = get_density(basisInfo,
-		       densityMatrixFull_spin,
-		       cutoff,
-		       nPrims2,
-		       rho2);
-  if(n1 < 0 || n2 < 0)
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "aborting SCF_unrestricted::output_density_images: (n1 < 0 || n2 < 0)");
-      throw "aborting SCF_unrestricted::output_density_images: (n1 < 0 || n2 < 0)";
-    }
-  nPrims1 = n1;
-  nPrims2 = n2;
-
-  // get min max coords for molecule
-  ergo_real minlist[3];
-  ergo_real maxlist[3];
-  int coordno;
-  for(coordno = 0; coordno < 3; coordno++)
-    {
-      minlist[coordno] =  888888888;
-      maxlist[coordno] = -888888888;
-    }
-  for(int i = 0; i < molecule.noOfAtoms; i++)
-    {
-      for(coordno = 0; coordno < 3; coordno++)
-	{
-	  ergo_real curr = molecule.atoms[i].coords[coordno];
-	  if(curr < minlist[coordno])
-	    minlist[coordno] = curr;
-	  if(curr > maxlist[coordno])
-	    maxlist[coordno] = curr;
-	}
-    }
-  // OK, now we have min max coords for molecule.
-  // Add some margin.
-  ergo_real margin = 4.4;
-  for(coordno = 0; coordno < 3; coordno++)
-    {
-      minlist[coordno] -= margin;
-      maxlist[coordno] += margin;
-    }
-
-  ergo_real boxWidthApprox = scfopts.output_density_images_boxwidth; //0.3;
-  int Nx = (int)((maxlist[0] - minlist[0]) / boxWidthApprox);
-  int Ny = (int)((maxlist[1] - minlist[1]) / boxWidthApprox);
-  int Nz = (int)((maxlist[2] - minlist[2]) / boxWidthApprox);
-
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "SCF_unrestricted::output_density_images: Nx Ny Nz = %5i %5i %5i", Nx, Ny, Nz);
-  
-  ergo_real boxWidth_x = (maxlist[0] - minlist[0]) / Nx;
-  ergo_real boxWidth_y = (maxlist[1] - minlist[1]) / Ny;
-  ergo_real boxWidth_z = (maxlist[2] - minlist[2]) / Nz;
-
-  FILE* f1 = fopen("density_image.txt", "wt");
-  FILE* f2 = fopen("spindensity_image.txt", "wt");
-  if(f1 == NULL || f2 == NULL)
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "aborting SCF_unrestricted::output_density_images: (f1 == NULL || f2 == NULL)");
-      throw "aborting SCF_unrestricted::output_density_images: (f1 == NULL || f2 == NULL)";
-    }
-
-  if(scfopts.image_view_axis == 0 || scfopts.image_view_axis == 3)
-    {
-      // z view
-      int ix, iy;
-      for(ix = 0; ix < Nx; ix++)
-	for(iy = 0; iy < Ny; iy++)
-	  {
-	    ergo_real minVect[3];
-	    ergo_real maxVect[3];
-	    minVect[0] = minlist[0] + ix * (maxlist[0] - minlist[0]) / Nx;
-	    maxVect[0] = minVect[0] + boxWidth_x;
-	    minVect[1] = minlist[1] + iy * (maxlist[1] - minlist[1]) / Ny;
-	    maxVect[1] = minVect[1] + boxWidth_y;
-	    minVect[2] = minlist[2];
-	    maxVect[2] = maxlist[2];
-	    ergo_real dens = integrate_density_in_box_2(nPrims1,
-							rho1,
-							minVect, 
-							maxVect);
-	    ergo_real spin = integrate_density_in_box_2(nPrims2,
-							rho2,
-							minVect, 
-							maxVect);
-	    fprintf(f1, "%11.7f %11.7f %11.7f\n", 
-		    (double)(minVect[0] + maxVect[0]) / 2,
-		    (double)(minVect[1] + maxVect[1]) / 2,
-		    (double)dens);
-	    fprintf(f2, "%11.7f %11.7f %11.7f\n", 
-		    (double)(minVect[0] + maxVect[0]) / 2,
-		    (double)(minVect[1] + maxVect[1]) / 2,
-		    (double)spin);
-	  } // END FOR ix iy
-    } // END IF z view
-  else if(scfopts.image_view_axis == 2)
-    {
-      // y view
-      int ix, iz;
-      for(ix = 0; ix < Nx; ix++)
-	for(iz = 0; iz < Nz; iz++)
-	  {
-	    ergo_real minVect[3];
-	    ergo_real maxVect[3];
-	    minVect[0] = minlist[0] + ix * (maxlist[0] - minlist[0]) / Nx;
-	    maxVect[0] = minVect[0] + boxWidth_x;
-	    minVect[1] = minlist[1];
-	    maxVect[1] = maxlist[1];
-	    minVect[2] = minlist[2] + iz * (maxlist[2] - minlist[2]) / Nz;
-	    maxVect[2] = minVect[2] + boxWidth_z;
-	    ergo_real dens = integrate_density_in_box_2(nPrims1,
-							rho1,
-							minVect, 
-							maxVect);
-	    ergo_real spin = integrate_density_in_box_2(nPrims2,
-							rho2,
-							minVect, 
-							maxVect);
-	    fprintf(f1, "%11.7f %11.7f %11.7f\n", 
-		    (double)(minVect[0] + maxVect[0]) / 2,
-		    (double)(minVect[2] + maxVect[2]) / 2,
-		    (double)dens);
-	    fprintf(f2, "%11.7f %11.7f %11.7f\n", 
-		    (double)(minVect[0] + maxVect[0]) / 2,
-		    (double)(minVect[2] + maxVect[2]) / 2,
-		    (double)spin);
-	  } // END FOR ix iy
-    } // END IF y view
-  else
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "aborting SCF_unrestricted::output_density_images: chosen image_view_axis not implemented.");
-    }
-  
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "SCF_unrestricted::output_density_images, closing files..");
-
-  fclose(f1);
-  fclose(f2);
-
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "SCF_unrestricted::output_density_images, files are now closed.");
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, "SCF_unrestricted::output_density_images finished OK.");
   timeMeter.print(LOG_AREA_SCF, "SCF_unrestricted::output_density_images");
 }
@@ -1195,15 +1060,15 @@ void SCF_unrestricted::do_spin_flip(int atomCount)
 	// get atom index for basis funcs i and j
 	int atomIndex_i = -1;
 	int atomIndex_j = -1;
-	for(int k = 0; k < molecule.noOfAtoms; k++)
+	for(int k = 0; k < molecule.getNoOfAtoms(); k++)
 	  {
 	    int iok = 1;
 	    int jok = 1;
 	    for(int coord = 0; coord < 3; coord++)
 	      {
-		if(std::fabs(molecule.atoms[k].coords[coord] - basisInfo.basisFuncList[i].centerCoords[coord]) > 1e-5)
+		if(std::fabs(molecule.getAtom(k).coords[coord] - basisInfo.basisFuncList[i].centerCoords[coord]) > 1e-5)
 		  iok = 0;
-		if(std::fabs(molecule.atoms[k].coords[coord] - basisInfo.basisFuncList[j].centerCoords[coord]) > 1e-5)
+		if(std::fabs(molecule.getAtom(k).coords[coord] - basisInfo.basisFuncList[j].centerCoords[coord]) > 1e-5)
 		  jok = 0;
 	      }
 	    if(iok == 1)
@@ -1252,12 +1117,6 @@ void SCF_unrestricted::output_csr_matrices_for_gao()
 }
 
 
-void SCF_unrestricted::do_electron_dynamics()
-{
-  throw "error: SCF_unrestricted::do_electron_dynamics not implemented.";
-}
-
-
 void SCF_unrestricted::write_diag_dens_to_file()
 {
   int n = basisInfo.noOfBasisFuncs;
@@ -1493,3 +1352,9 @@ void SCF_unrestricted::disturb_dens_matrix_exact(ergo_real subspaceError)
 {
   throw "SCF_unrestricted::disturb_dens_matrix_exact() not implemented";
 }
+
+void SCF_unrestricted::compute_gradient_fixeddens()
+{
+  throw "SCF_unrestricted::compute_gradient_fixeddens() not implemented";  
+}
+
