@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -55,6 +55,7 @@
 #include "basis_func_pair_list_1el.h"
 #include "integral_matrix_wrappers.h"
 #include "units.h"
+#include "densitymanager.h"
 
 
 /* Flag for skipping DFT stuff, used in QD precision tests. */
@@ -535,8 +536,8 @@ compute_h_core_matrix_simple_dense(const IntegralInfo& integralInfo,
   std::vector<ergo_real> A(n*n);
   if(compute_h_core_matrix_full(integralInfo,
 				basisInfo,
-				molecule.noOfAtoms,
-				molecule.atoms,
+				molecule.getNoOfAtoms(),
+				molecule.getAtomListPtr(),
 				&A[0],
 				threshold_integrals_1el) != 0)
     {
@@ -587,7 +588,7 @@ compute_h_core_matrix_sparse(const IntegralInfo& integralInfo,
 {
   int n = basisInfo.noOfBasisFuncs;
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, 
-	    "entering compute_h_core_matrix_sparse, nAtoms = %i, n = %i, threshold = %g", molecule.noOfAtoms, n, (double)threshold_integrals_1el);
+	    "entering compute_h_core_matrix_sparse, nAtoms = %i, n = %i, threshold = %g", molecule.getNoOfAtoms(), n, (double)threshold_integrals_1el);
 
   Util::TimeMeter timeMeterTot;
   
@@ -637,6 +638,71 @@ compute_h_core_matrix_sparse(const IntegralInfo& integralInfo,
   // Write to file and read again to reduce memory fragmentation.
   H_core_Matrix_sparse.writeToFile();
   H_core_Matrix_sparse.readFromFile();
+
+  if(create_dipole_mtx == 1) {
+    // Output dipole matrices in mtx format
+    if ( calculation_identifier == 0 )
+      throw "calculation_identifier == 0 when create_dipole_mtx == 1";
+    if ( method_and_basis_set == 0 )
+      throw "method_and_basis_set == 0 when create_dipole_mtx == 1";
+    if ( inversePermutationHML  == 0 )
+      throw "inversePermutationHML == 0 when create_dipole_mtx == 1";	  
+    {
+      // dipole matrix X
+      std::stringstream id_ss;
+      id_ss << *calculation_identifier << " - dipole matrix X";
+      symmMatrix fieldMatrix(H_core_Matrix_sparse);
+      fieldMatrix.clear();
+      if(compute_operator_matrix_sparse_symm(basisInfo, 
+					     1, 0, 0, 
+					     fieldMatrix, 
+					     permutationHML) != 0)
+	return -1;
+      do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Creating mtx file for dipole matrix X");
+      write_matrix_in_matrix_market_format( fieldMatrix, 
+					    *inversePermutationHML, 
+					    "dipole_matrix_x", 
+					    id_ss.str(), 
+					    *method_and_basis_set );	
+    }
+    {
+      // dipole matrix Y
+      std::stringstream id_ss;
+      id_ss << *calculation_identifier << " - dipole matrix Y";
+      symmMatrix fieldMatrix(H_core_Matrix_sparse);
+      fieldMatrix.clear();
+      if(compute_operator_matrix_sparse_symm(basisInfo, 
+					     0, 1, 0, 
+					     fieldMatrix, 
+					     permutationHML) != 0)
+	return -1;
+      do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Creating mtx file for dipole matrix Y");
+      write_matrix_in_matrix_market_format( fieldMatrix, 
+					    *inversePermutationHML, 
+					    "dipole_matrix_y", 
+					    id_ss.str(), 
+					    *method_and_basis_set );	
+    }
+    {
+      // dipole matrix Z
+      std::stringstream id_ss;
+      id_ss << *calculation_identifier << " - dipole matrix Z";
+      symmMatrix fieldMatrix(H_core_Matrix_sparse);
+      fieldMatrix.clear();
+      if(compute_operator_matrix_sparse_symm(basisInfo, 
+					     0, 0, 1, 
+					     fieldMatrix, 
+					     permutationHML) != 0)
+	return -1;
+      do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Creating mtx file for dipole matrix Z");
+      write_matrix_in_matrix_market_format( fieldMatrix, 
+					    *inversePermutationHML, 
+					    "dipole_matrix_z", 
+					    id_ss.str(), 
+					    *method_and_basis_set );	
+    }
+  } // end if(create_dipole_mtx == 1)
+
   
   // Add contributions from electric field, if any
   if(electric_field_x != 0)
@@ -650,22 +716,6 @@ compute_h_core_matrix_sparse(const IntegralInfo& integralInfo,
 					     fieldMatrix, 
 					     permutationHML) != 0)
 	return -1;
-      if(create_dipole_mtx == 1) {
-	if ( calculation_identifier == 0 )
-	  throw "calculation_identifier == 0 when create_dipole_mtx == 1";
-	if ( method_and_basis_set == 0 )
-	  throw "method_and_basis_set == 0 when create_dipole_mtx == 1";
-	if ( inversePermutationHML  == 0 )
-	  throw "inversePermutationHML == 0 when create_dipole_mtx == 1";	
-	std::stringstream id_ss;
-	id_ss << *calculation_identifier << " - dipole matrix X";
-	do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Creating mtx file for dipole matrix X");
-	write_matrix_in_matrix_market_format( fieldMatrix, 
-					      *inversePermutationHML, 
-					      "dipole_matrix_x", 
-					      id_ss.str(), 
-					      *method_and_basis_set );	
-      }
       H_core_Matrix_sparse += 
 	(ergo_real)(-1.0) * electric_field_x * fieldMatrix;
     }
@@ -680,22 +730,6 @@ compute_h_core_matrix_sparse(const IntegralInfo& integralInfo,
 					     fieldMatrix, 
 					     permutationHML) != 0)
 	return -1;
-      if(create_dipole_mtx == 1) {
-	if ( calculation_identifier == 0 )
-	  throw "calculation_identifier == 0 when create_dipole_mtx == 1";
-	if ( method_and_basis_set == 0 )
-	  throw "method_and_basis_set == 0 when create_dipole_mtx == 1";
-	if ( inversePermutationHML  == 0 )
-	  throw "inversePermutationHML == 0 when create_dipole_mtx == 1";	
-	std::stringstream id_ss;
-	id_ss << *calculation_identifier << " - dipole matrix Y";
-	do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Creating mtx file for dipole matrix Y");
-	write_matrix_in_matrix_market_format( fieldMatrix, 
-					      *inversePermutationHML, 
-					      "dipole_matrix_y", 
-					      id_ss.str(), 
-					      *method_and_basis_set );	
-      }
       H_core_Matrix_sparse += 
 	(ergo_real)(-1.0) * electric_field_y * fieldMatrix;
     }
@@ -710,34 +744,18 @@ compute_h_core_matrix_sparse(const IntegralInfo& integralInfo,
 					     fieldMatrix, 
 					     permutationHML) != 0)
 	return -1;
-      if(create_dipole_mtx == 1) {
-	if ( calculation_identifier == 0 )
-	  throw "calculation_identifier == 0 when create_dipole_mtx == 1";
-	if ( method_and_basis_set == 0 )
-	  throw "method_and_basis_set == 0 when create_dipole_mtx == 1";
-	if ( inversePermutationHML  == 0 )
-	  throw "inversePermutationHML == 0 when create_dipole_mtx == 1";	
-	std::stringstream id_ss;
-	id_ss << *calculation_identifier << " - dipole matrix Z";
-	do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Creating mtx file for dipole matrix Z");
-	write_matrix_in_matrix_market_format( fieldMatrix, 
-					      *inversePermutationHML, 
-					      "dipole_matrix_z", 
-					      id_ss.str(), 
-					      *method_and_basis_set );	
-      }
       H_core_Matrix_sparse += 
 	(ergo_real)(-1.0) * electric_field_z * fieldMatrix;
     }
 
   // Add contribution from extra charges, if any.
-  if(extraCharges.noOfAtoms > 0) {
-    do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Using 'extra charges', extraCharges.noOfAtoms = %d", extraCharges.noOfAtoms);
+  if(extraCharges.getNoOfAtoms() > 0) {
+    do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Using 'extra charges', extraCharges.noOfAtoms = %d", extraCharges.getNoOfAtoms());
     ergo_real sumCharge = 0;
-    ergo_real maxCharge = extraCharges.atoms[0].charge;
-    ergo_real minCharge = extraCharges.atoms[0].charge;
-    for(int i = 0; i < extraCharges.noOfAtoms; i++) {
-      ergo_real currCharge = extraCharges.atoms[i].charge;
+    ergo_real maxCharge = extraCharges.getAtom(0).charge;
+    ergo_real minCharge = extraCharges.getAtom(0).charge;
+    for(int i = 0; i < extraCharges.getNoOfAtoms(); i++) {
+      ergo_real currCharge = extraCharges.getAtom(i).charge;
       sumCharge += currCharge;
       if(currCharge > maxCharge)
 	maxCharge = currCharge;
@@ -749,16 +767,16 @@ compute_h_core_matrix_sparse(const IntegralInfo& integralInfo,
     ergo_real minDist = std::numeric_limits<ergo_real>::max();
     ergo_real minDistCharge1 = 0;
     ergo_real minDistCharge2 = 0;
-    for(int i = 0; i < extraCharges.noOfAtoms; i++)
-      for(int j = 0; j < molecule.noOfAtoms; j++) {
-	ergo_real dx = extraCharges.atoms[i].coords[0] - molecule.atoms[j].coords[0];
-	ergo_real dy = extraCharges.atoms[i].coords[1] - molecule.atoms[j].coords[1];
-	ergo_real dz = extraCharges.atoms[i].coords[2] - molecule.atoms[j].coords[2];
+    for(int i = 0; i < extraCharges.getNoOfAtoms(); i++)
+      for(int j = 0; j < molecule.getNoOfAtoms(); j++) {
+	ergo_real dx = extraCharges.getAtom(i).coords[0] - molecule.getAtom(j).coords[0];
+	ergo_real dy = extraCharges.getAtom(i).coords[1] - molecule.getAtom(j).coords[1];
+	ergo_real dz = extraCharges.getAtom(i).coords[2] - molecule.getAtom(j).coords[2];
 	ergo_real dist = std::sqrt(dx*dx+dy*dy+dz*dz);
 	if(dist < minDist) {
 	  minDist = dist;
-	  minDistCharge1 = molecule.atoms[j].charge;
-	  minDistCharge2 = extraCharges.atoms[i].charge;
+	  minDistCharge1 = molecule.getAtom(j).charge;
+	  minDistCharge2 = extraCharges.getAtom(i).charge;
 	}
       }
     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Min dist between any 'extra charge' atom and any atom in main molecule: %9.5f a.u. = %9.5f Angstrom", 
@@ -790,6 +808,33 @@ compute_h_core_matrix_sparse(const IntegralInfo& integralInfo,
 }
 
 
+ergo_real
+get_electron_nuclear_attraction_energy(const IntegralInfo& integralInfo,
+				       const Molecule& molecule,
+				       const BasisInfoStruct& basisInfo,
+				       const symmMatrix & D,
+				       ergo_real threshold_integrals_1el,
+				       mat::SizesAndBlocks const & matrix_size_block_info,
+				       std::vector<int> const & permutationHML)
+{
+  // Get V
+  symmMatrix V;
+  V.resetSizesAndBlocks(matrix_size_block_info,
+			matrix_size_block_info);
+  ergo_real boxSize = 10.0;
+  if(compute_V_sparse(basisInfo,
+		      integralInfo, 
+		      molecule,
+		      threshold_integrals_1el,
+		      boxSize,
+		      V,
+		      permutationHML) != 0)
+    {
+      throw "error in compute_V_sparse";
+    }
+  return symmMatrix::trace_ab(D, V);
+}
+
 
 
 /** Saves specified symmetic matrix to a file of specified name.
@@ -1060,6 +1105,26 @@ write_basis_func_coord_file(const BasisInfoStruct & basisInfo)
 }
 
 
+int
+write_2el_integral_m_file(const BasisInfoStruct & basisInfo, const IntegralInfo & integralInfo)
+{
+  int n = basisInfo.noOfBasisFuncs;
+  FILE* f = fopen("setup_twoel_integrals.m", "wt");
+  if(f == NULL)
+    throw "write_2el_integral_m_file, in fopen.";  
+  fprintf(f, "twoel_integrals = zeros(%2d, %2d, %2d, %2d);\n", n, n, n, n);
+  for(int a = 0; a < n; a++)
+    for(int b = 0; b < n; b++)
+      for(int c = 0; c < n; c++)
+	for(int d = 0; d < n; d++) {
+	  ergo_real integralValue = do_2e_integral(a, b, c, d, basisInfo, integralInfo);
+	  fprintf(f, "twoel_integrals(%2d, %2d, %2d, %2d) = %15.11f;\n", a+1, b+1, c+1, d+1, (double)integralValue);
+	}
+  fclose(f);
+  return 0;
+}
+
+
 
 static int
 get_2e_matrix_and_energy_simple_HF_sparse(const BasisInfoStruct& basisInfo,
@@ -1615,7 +1680,7 @@ get_2e_matrix_and_energy_sparse(const BasisInfoStruct & basisInfo,
       ergo_real threshold_JK = J_K_params.threshold_J;
       if(J_K_params.threshold_K < threshold_JK)
 	threshold_JK = J_K_params.threshold_K;
-      if(compute_2e_matrix_list_explicit(basisInfo, &integralInfo, &twoelMatrixG, &densityMatrixFull, 1, threshold_JK) != 0)
+      if(compute_2e_matrix_list_explicit(basisInfo, integralInfo, &twoelMatrixG, &densityMatrixFull, 1, threshold_JK) != 0)
 	{
 	  do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_2e_matrix_list_explicit");
 	  return -1;
@@ -1705,7 +1770,7 @@ get_2e_matrix_and_energy_sparse(const BasisInfoStruct & basisInfo,
       if(J_K_params.use_differential_density == 1)
 	{
 	  if(compute_2e_matrix_list_difden(basisInfo,
-					   &integralInfo,
+					   integralInfo,
 					   CAM_params,
 					   &twoelMatrix_full,
 					   &densityMatrix_full,
@@ -1719,7 +1784,7 @@ get_2e_matrix_and_energy_sparse(const BasisInfoStruct & basisInfo,
       else
 	{
 	  if(compute_2e_matrix_list(basisInfo,
-				    &integralInfo,
+				    integralInfo,
 				    CAM_params,
 				    &twoelMatrix_full,
 				    &densityMatrix_full,
@@ -1735,7 +1800,7 @@ get_2e_matrix_and_energy_sparse(const BasisInfoStruct & basisInfo,
     {
       // Get J
       if(compute_2e_matrix_coulomb(basisInfo, basisInfoDensFit,
-                                   &integralInfo, 
+                                   integralInfo, 
 				   twoelMatrix_full, 
 				   densityMatrix_full, 
 				   J_K_params,
@@ -1750,7 +1815,7 @@ get_2e_matrix_and_energy_sparse(const BasisInfoStruct & basisInfo,
 	  ergo_real* K   = new ergo_real[n*n];
 	  // Get K
 	  if(compute_2e_matrix_exchange(basisInfo, 
-					&integralInfo, 
+					integralInfo, 
 					CAM_params,
 					K, 
 					densityMatrix_full,
@@ -1933,7 +1998,7 @@ get_2e_matrices_and_energy_sparse_unrestricted(const BasisInfoStruct & basisInfo
 	    threshold_JK = J_K_params.threshold_K;
 
 	  if(compute_JK_single_box(basisInfo,
-				   &integralInfo,
+				   integralInfo,
 				   J_list[0],
 				   K_list[0],
 				   D_list[0],
@@ -1943,7 +2008,7 @@ get_2e_matrices_and_energy_sparse_unrestricted(const BasisInfoStruct & basisInfo
 	      return -1;
 	    }
 	  if(compute_JK_single_box(basisInfo,
-				   &integralInfo,
+				   integralInfo,
 				   J_list[1],
 				   K_list[1],
 				   D_list[1],
@@ -2357,8 +2422,8 @@ compute_dipole_moment_onecoord(const symmMatrix & densityMatrix,
   ergo_real density_contrib = symmMatrix::trace_ab(densityMatrix, opMatrix);
   // Now compute contribution from nuclei.
   ergo_real nuclear_contrib = 0;
-  for(int i = 0; i < molecule.noOfAtoms; i++)
-    nuclear_contrib += molecule.atoms[i].charge * molecule.atoms[i].coords[coordIdx];
+  for(int i = 0; i < molecule.getNoOfAtoms(); i++)
+    nuclear_contrib += molecule.getAtom(i).charge * molecule.getAtom(i).coords[coordIdx];
   ergo_real dipole_moment_oneCoord = nuclear_contrib - density_contrib;
   return dipole_moment_oneCoord;
 }
@@ -2390,7 +2455,7 @@ get_mulliken_charges(const symmMatrix & densityMatrix,
 		     const Molecule& molecule,
 		     std::vector<ergo_real> & resultVector)
 {
-  int nAtoms = molecule.noOfAtoms;
+  int nAtoms = molecule.getNoOfAtoms();
   resultVector.resize(nAtoms);
   for(int i = 0; i < nAtoms; i++)
     resultVector[i] = 0;
@@ -2401,9 +2466,9 @@ get_mulliken_charges(const symmMatrix & densityMatrix,
     // Check if this basis function is near any atom.
     int foundAtomIndex = -1;
     for(int k = 0; k < nAtoms; k++) {
-      ergo_real dx = molecule.atoms[k].coords[0] - basisInfo.basisFuncList[i].centerCoords[0];
-      ergo_real dy = molecule.atoms[k].coords[1] - basisInfo.basisFuncList[i].centerCoords[1];
-      ergo_real dz = molecule.atoms[k].coords[2] - basisInfo.basisFuncList[i].centerCoords[2];
+      ergo_real dx = molecule.getAtom(k).coords[0] - basisInfo.basisFuncList[i].centerCoords[0];
+      ergo_real dy = molecule.getAtom(k).coords[1] - basisInfo.basisFuncList[i].centerCoords[1];
+      ergo_real dz = molecule.getAtom(k).coords[2] - basisInfo.basisFuncList[i].centerCoords[2];
       ergo_real distance = std::sqrt(dx*dx + dy*dy + dz*dz);
       if(distance < 1e-4)
 	foundAtomIndex = k;
@@ -2460,14 +2525,14 @@ do_mulliken_atomic_charges(const symmMatrix & densityMatrix,
   // First get vector with summed electron charge for all atoms.
   std::vector<ergo_real> electronChargeSums;
   get_mulliken_charges(densityMatrix, S_symm, basisInfo, matrix_size_block_info, permutationHML, inversePermutationHML, molecule, electronChargeSums);
-  int nAtoms = molecule.noOfAtoms;
+  int nAtoms = molecule.getNoOfAtoms();
   ergo_real chargeSum = 0;
   for(int i = 0; i < nAtoms; i++)
     chargeSum += electronChargeSums[i];
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Sum of Mulliken charges (electrons): %15.7f", (double)chargeSum);
   std::vector<ergo_real> atomicCharges(nAtoms);
   for(int i = 0; i < nAtoms; i++)
-    atomicCharges[i] = molecule.atoms[i].charge - electronChargeSums[i];
+    atomicCharges[i] = molecule.getAtom(i).charge - electronChargeSums[i];
   for(int i = 0; i < nAtoms; i++)
     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Mulliken charge of atom %d = %15.7f", i, (double)atomicCharges[i]);    
   chargeSum = 0;
@@ -2487,7 +2552,7 @@ do_mulliken_spin_densities(const symmMatrix & spinDensityMatrix,
 {
   std::vector<ergo_real> mullikenCharges;
   get_mulliken_charges(spinDensityMatrix, S_symm, basisInfo, matrix_size_block_info, permutationHML, inversePermutationHML, molecule, mullikenCharges);
-  int nAtoms = molecule.noOfAtoms;
+  int nAtoms = molecule.getNoOfAtoms();
   for(int i = 0; i < nAtoms; i++)
     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Mulliken spin density of atom %d = %15.7f", i, (double)mullikenCharges[i]);
   ergo_real chargeSum = 0;
@@ -2496,6 +2561,218 @@ do_mulliken_spin_densities(const symmMatrix & spinDensityMatrix,
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Sum of Mulliken atomic spin densities: %15.7f", (double)chargeSum);  
 }
 
+static int write_gcube_file_header(FILE* f, 
+				   const char* firstLine,
+				   const Molecule& m, 
+				   double originX, double originY, double originZ, 
+				   int NX, double incrX,
+				   int NY, double incrY,
+				   int NZ, double incrZ) {
+  fprintf(f, "%s\n\n", firstLine);
+  fprintf(f, "%d  %9.5f  %9.5f  %9.5f\n", m.getNoOfAtoms(), originX, originY, originZ);
+  fprintf(f, "%d  %9.5f    0.0    0.0\n", NX, incrX);
+  fprintf(f, "%d    0.0  %9.5f    0.0\n", NY, incrY);
+  fprintf(f, "%d    0.0    0.0  %9.5f\n", NZ, incrZ);
+  for(int i = 0; i < m.getNoOfAtoms(); i++)
+    fprintf(f, "%d 0.0 %9.5f %9.5f %9.5f\n", (int)m.getAtom(i).charge, (double)m.getAtom(i).coords[0], (double)m.getAtom(i).coords[1], (double)m.getAtom(i).coords[2]);
+  return 0;
+}
+
+void 
+do_density_images(const BasisInfoStruct & basisInfo,
+		  const Molecule& molecule,
+		  const ergo_real* densityMatrixFull_tot, 
+		  const ergo_real* densityMatrixFull_spin,
+		  double output_density_images_boxwidth)
+{
+
+  int nPrims1 = 0;
+  int nPrims2 = 0;
+  
+  DistributionSpecStruct* rho1 = NULL;
+  DistributionSpecStruct* rho2 = NULL;
+  ergo_real cutoff = 1e-7;
+
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "do_density_images, calling get_density() with cutoff = %8.4g.", cutoff);
+
+  nPrims1 = get_no_of_primitives_for_density(cutoff, densityMatrixFull_tot , basisInfo);
+  nPrims2 = get_no_of_primitives_for_density(cutoff, densityMatrixFull_spin, basisInfo);
+  rho1 = new DistributionSpecStruct[nPrims1];
+  rho2 = new DistributionSpecStruct[nPrims2];
+
+  Util::TimeMeter timeMeterGetDens;
+  int n1 = get_density(basisInfo,
+		       densityMatrixFull_tot,
+		       cutoff,
+		       nPrims1,
+		       rho1);      
+  int n2 = get_density(basisInfo,
+		       densityMatrixFull_spin,
+		       cutoff,
+		       nPrims2,
+		       rho2);
+  if(n1 < 0 || n2 < 0)
+    {
+      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "aborting do_density_images(): (n1 < 0 || n2 < 0)");
+      throw "aborting do_density_images(): (n1 < 0 || n2 < 0)";
+    }
+  nPrims1 = n1;
+  nPrims2 = n2;
+  timeMeterGetDens.print(LOG_AREA_SCF, "get_density() calls");
+
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "do_density_images, after get_density() calls: nPrims1 = %d, nPrims2 = %d.", nPrims1, nPrims2);
+
+  // get min max coords for molecule
+  ergo_real minlist[3];
+  ergo_real maxlist[3];
+  int coordno;
+  for(coordno = 0; coordno < 3; coordno++)
+    {
+      minlist[coordno] =  888888888;
+      maxlist[coordno] = -888888888;
+    }
+  for(int i = 0; i < molecule.getNoOfAtoms(); i++)
+    {
+      for(coordno = 0; coordno < 3; coordno++)
+	{
+	  ergo_real curr = molecule.getAtom(i).coords[coordno];
+	  if(curr < minlist[coordno])
+	    minlist[coordno] = curr;
+	  if(curr > maxlist[coordno])
+	    maxlist[coordno] = curr;
+	}
+    }
+  // OK, now we have min max coords for molecule.
+  // Add some margin.
+  ergo_real margin = 4.4;
+  for(coordno = 0; coordno < 3; coordno++)
+    {
+      minlist[coordno] -= margin;
+      maxlist[coordno] += margin;
+    }
+
+  ergo_real boxWidthApprox = output_density_images_boxwidth; //0.3;
+  int Nx = (int)((maxlist[0] - minlist[0]) / boxWidthApprox);
+  int Ny = (int)((maxlist[1] - minlist[1]) / boxWidthApprox);
+  int Nz = (int)((maxlist[2] - minlist[2]) / boxWidthApprox);
+
+  // Make sure Nx Ny Nz are odd numbers (this seems to be required for the electrostatic potential computation feature in Gabedit).
+  if(Nx % 2 == 0) Nx++;
+  if(Ny % 2 == 0) Ny++;
+  if(Nz % 2 == 0) Nz++;
+
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "do_density_images: Nx Ny Nz = %5i %5i %5i", Nx, Ny, Nz);
+  
+  ergo_real boxWidth_x = (maxlist[0] - minlist[0]) / Nx;
+  ergo_real boxWidth_y = (maxlist[1] - minlist[1]) / Ny;
+  ergo_real boxWidth_z = (maxlist[2] - minlist[2]) / Nz;
+
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "do_density_images, preparing to write Gabedit gcube files.");
+
+  // Create Gabedit "gcube" density files.
+  FILE* f1 = fopen("density.gcube", "wt");
+  FILE* f2 = fopen("spindensity.gcube", "wt");
+  if(f1 == NULL || f2 == NULL)
+    {
+      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "aborting do_density_images: (f1 == NULL || f2 == NULL)");
+      throw "aborting do_density_images: (f1 == NULL || f2 == NULL)";
+    }
+  double originX = minlist[0];
+  double originY = minlist[1];
+  double originZ = minlist[2];
+  double dx = (maxlist[0] - minlist[0]) / Nx;
+  double dy = (maxlist[1] - minlist[1]) / Ny;
+  double dz = (maxlist[2] - minlist[2]) / Nz;
+  // Shift origin coordinates by half of the box width in each direction (it seems Gabedit is interpreting the values in that way).
+  originX += 0.5 * dx;
+  originY += 0.5 * dy;
+  originZ += 0.5 * dz;
+  write_gcube_file_header(f1, 
+			  "Density file in gcube format, generated by the Ergo program.",
+			  molecule, 
+			  originX, originY, originZ, 
+			  Nx, dx,
+			  Ny, dy,
+			  Nz, dz);
+  write_gcube_file_header(f2, 
+			  "Spin density file in gcube format, generated by the Ergo program.",
+			  molecule, 
+			  originX, originY, originZ, 
+			  Nx, dx,
+			  Ny, dy,
+			  Nz, dz);
+  // Now compute density values in all needed boxes.
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "do_density_images, computing density values needed for Gabedit gcube files.");
+  Util::TimeMeter timeMeterGetDensValues;
+  std::vector< std::vector< std::vector<double> > > vector_dens(Nx);
+  std::vector< std::vector< std::vector<double> > > vector_spin(Nx);
+#ifdef _OPENMP
+#pragma omp parallel for
+#endif
+  for(int ix = 0; ix < Nx; ix++) {
+    vector_dens[ix].resize(Ny);
+    vector_spin[ix].resize(Ny);
+    for(int iy = 0; iy < Ny; iy++) {
+      vector_dens[ix][iy].resize(Nz);
+      vector_spin[ix][iy].resize(Nz);
+      for(int iz = 0; iz < Nz; iz++) {
+	ergo_real minVect[3];
+	ergo_real maxVect[3];
+	minVect[0] = minlist[0] + ix * (maxlist[0] - minlist[0]) / Nx;
+	maxVect[0] = minVect[0] + boxWidth_x;
+	minVect[1] = minlist[1] + iy * (maxlist[1] - minlist[1]) / Ny;
+	maxVect[1] = minVect[1] + boxWidth_y;
+	minVect[2] = minlist[2] + iz * (maxlist[2] - minlist[2]) / Nz;
+	maxVect[2] = minVect[2] + boxWidth_z;
+	ergo_real dens = integrate_density_in_box_2(nPrims1,
+						    rho1,
+						    minVect, 
+						    maxVect);
+	ergo_real spin = integrate_density_in_box_2(nPrims2,
+						    rho2,
+						    minVect, 
+						    maxVect);
+	vector_dens[ix][iy][iz] = dens;
+	vector_spin[ix][iy][iz] = spin;
+      } // END FOR iz
+    } // END FOR iy
+  } // END FOR ix
+  timeMeterGetDensValues.print(LOG_AREA_SCF, "getting density values");
+  // Now write to files.
+  Util::TimeMeter timeMeterWriteFiles;
+  ergo_real sum_dens = 0;
+  ergo_real sum_spin = 0;
+  for(int ix = 0; ix < Nx; ix++)
+    for(int iy = 0; iy < Ny; iy++) {
+      int count = 0;
+      for(int iz = 0; iz < Nz; iz++) {
+	ergo_real dens = vector_dens[ix][iy][iz];
+	ergo_real spin = vector_spin[ix][iy][iz];
+	sum_dens += dens;
+	sum_spin += spin;
+	ergo_real volume = dx*dy*dz;
+	fprintf(f1, " %9.5f", (double)(dens / volume));
+	fprintf(f2, " %9.5f", (double)(spin / volume));
+	count++;
+	if(count % 6 == 0) {
+	  fprintf(f1, "\n");
+	  fprintf(f2, "\n");
+	}
+      }
+      if(count % 6 != 0) {
+	fprintf(f1, "\n");
+	fprintf(f2, "\n");
+      }
+    }
+  timeMeterWriteFiles.print(LOG_AREA_SCF, "writing to gcube files");
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "sum_dens = %9.5f, sum_spin = %9.5f", sum_dens, sum_spin);
+
+  fclose(f1);
+  fclose(f2);
+
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "do_density_images done, Gabedit gcube files written.");
+}
+
 
 void
 get_hf_weight_and_cam_params(int use_dft, 
