@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -128,7 +128,7 @@ load_density_and_project_full(const char *densityFileName,
   }
 
   // get matrix Sinv : inverse of main overlap matrix
-  ergo_real* Sinv = ergo_new(n*n, ergo_real);
+  //  ergo_real* Sinv = ergo_new(n*n, ergo_real);
 
   
   do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "error: get_inverse_of_posdef_symm_matrix not implemented.");
@@ -140,93 +140,8 @@ load_density_and_project_full(const char *densityFileName,
       return -1;
   }
   */
-
-  // get matrix B = R * Sinv
-  ergo_real* B = ergo_new(n*n_sg, ergo_real);
-  multiply_matrices_general(n_sg, n, n, n, R, Sinv, B);
-
-  
-  // Now that we have B we do not need R or Sinv any more
-  ergo_free(R);
-  R = NULL;
-  ergo_free(Sinv);
-  Sinv = NULL;
-
-  ergo_real* PB  = ergo_new(n*n_sg, ergo_real);
-  
-  output_current_memory_usage(2, "While doing projection of starting guess (1).");
-
-  // Now calculate BT * P * B for each density matrix
-  for(int i = 0; i < noOfDensityMatrices; i++)
-    {
-      ergo_real* P = densityMatrixListForStartingGuess[i];
-
-      // get matrix PB = P * B
-      multiply_matrices_general(n_sg, n_sg, n_sg, n, P, B, PB);
-
-      // get starting guess density matrix D = BT * P * B = BT * PB
-      multiply_matrices_general_T_1(n_sg, n, n_sg, n, B, PB, densityMatrixList[i]);
-
-    }
-
-  ergo_free(PB);
-  PB = NULL;
-  ergo_free(B);
-  B = NULL;
-
-  // Now we do not need densityMatrixListForStartingGuess any more
-  for(int i = 0; i < noOfDensityMatrices; i++)
-    {
-      ergo_free(densityMatrixListForStartingGuess[i]);
-      densityMatrixListForStartingGuess[i] = NULL;
-    }
-
-
-  if(do_purification == 1)
-    {
-      ergo_real* SDS = ergo_new(n*n, ergo_real);
-
-      output_current_memory_usage(2, "While doing projection of starting guess (2).");
-
-      for(int i = 0; i < noOfDensityMatrices; i++)
-	{
-	  // Do purification to force idempotency and correct trace
-	  multiply3matrices(n, S, densityMatrixList[i], S, SDS);
-	  for(long j = 0; j < n*n; j++)
-	    SDS[j] *= -1;
-	  ergo_real factor = 2;
-	  if(noOfDensityMatrices == 2)
-	    factor = 1;
-	  if(noOfElectronsList == NULL)
-	    return -1;
-	  int noOfOccupiedOrbs = noOfElectronsList[i];
-	  if(noOfDensityMatrices == 1)
-	    noOfOccupiedOrbs /= 2;
-	  do_output(LOG_CAT_INFO, LOG_AREA_MAIN, 
-		    "calling get_dens_from_fock_full for SDS to force "
-		    "idempotency and correct trace of starting guess");
-	  Util::TimeMeter timeMeterDensFromF;
-	  if(get_dens_from_fock_full(n, 
-				     noOfOccupiedOrbs, 
-				     densityMatrixList[i], 
-				     SDS, 
-				     S, 
-				     factor,
-				     electronic_temperature) != 0)
-	    {
-	      do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, 
-			"error in get_dens_from_fock");
-	      return -1;
-	    }
-	  timeMeterDensFromF.print(LOG_AREA_MAIN, "get_dens_from_fock_full"); 
-	  do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "get_dens_from_fock_full finished OK.");
-	} // END FOR i
-      ergo_free(SDS);
-    } // END IF (do_purification == 1)
-  
-  delete basisInfoStartingGuess;
-  ergo_free(S);
-  return 0;
+  /* ELIAS NOTE 2011-09-15: 
+     Remainder of this routine removed since it could anyway not be used since get_inverse_of_posdef_symm_matrix was not implemented. */
 }
 
 
@@ -520,6 +435,7 @@ load_density_and_project_sparse(const char *densityFileName,
       SDS.writeToFile();
 
       std::map<std::string, double> puri_stats;
+      ergo_real electronicEntropyTerm_dummy = 0;
       if(get_dens_from_fock_general(n,
 				    noOfOccupiedOrbs,
 				    use_diagonalization,
@@ -527,6 +443,7 @@ load_density_and_project_sparse(const char *densityFileName,
 				    electronic_temperature,
 				    *densityMatrixList[i],
 				    factor,
+				    electronicEntropyTerm_dummy,
 				    SDS, 
 				    homoInterval_dummy1,
 				    lumoInterval_dummy1,
@@ -549,7 +466,8 @@ load_density_and_project_sparse(const char *densityFileName,
 				    puri_stats,
 				    0, // no sparsity investigation
 				    1, // dummy 1 here (no investigation)
-				    0  // no comparison to simple purification
+				    0, // no comparison to simple purification
+				    0  // purification mmul tests
 				    ) != 0)
 	{
 	  do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in get_dens_from_fock_general.");
