@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -53,16 +53,16 @@ compute_V_sparse(const BasisInfoStruct& basisInfo,
 
   // Get maxCharge
   ergo_real maxCharge = 0;
-  for(int i = 0; i < molecule.noOfAtoms; i++)
+  for(int i = 0; i < molecule.getNoOfAtoms(); i++)
     {
-      ergo_real currCharge = molecule.atoms[i].charge;
+      ergo_real currCharge = molecule.getAtom(i).charge;
       if(currCharge > maxCharge)
 	maxCharge = currCharge;
     }
 
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, 
-	    "compute_V_sparse, molecule.noOfAtoms = %5i, maxCharge = %6.2f", 
-	    molecule.noOfAtoms, (double)maxCharge);
+	    "compute_V_sparse, no of atoms = %5i, maxCharge = %6.2f", 
+	    molecule.getNoOfAtoms(), (double)maxCharge);
 
   int noOfBasisFuncIndexPairs = get_basis_func_pair_list_1el(basisInfo,
 							     threshold,
@@ -287,7 +287,7 @@ compute_J_by_boxes_sparse(const BasisInfoStruct& basisInfo,
 
   std::vector<basis_func_index_pair_struct> basisFuncIndexPairList;
   int noOfBasisFuncIndexPairs = get_basis_func_pair_list_2el(basisInfo,
-							     &integralInfo,
+							     integralInfo,
 							     J_K_params.threshold_J,
 							     maxDensityMatrixElement,
 							     basisFuncIndexPairList);
@@ -322,7 +322,7 @@ compute_J_by_boxes_sparse(const BasisInfoStruct& basisInfo,
 
   output_current_memory_usage(LOG_AREA_SCF, "Before calling compute_J_by_boxes_linear");
   if(compute_J_by_boxes_linear(basisInfo,
-			       &integralInfo,
+			       integralInfo,
 			       J_K_params,
 			       &basisFuncIndexPairList[0],
 			       noOfBasisFuncIndexPairs,
@@ -493,7 +493,7 @@ compute_K_by_boxes_sparse(const BasisInfoStruct& basisInfo,
   // set up CSR K matrix for result
   // note that this call will allocate memory, we must call ergo_CSR_destroy later.
   if(create_CSR_for_K(basisInfo,
-		      &integralInfo,
+		      integralInfo,
 		      J_K_params,
 		      &dens_CSR,
 		      &K_CSR,
@@ -507,7 +507,7 @@ compute_K_by_boxes_sparse(const BasisInfoStruct& basisInfo,
   
   output_current_memory_usage(LOG_AREA_SCF, "Before calling compute_K_by_boxes");
   if(compute_K_by_boxes(basisInfo,
-			&integralInfo,
+			integralInfo,
 			CAM_params,
 			J_K_params,
 			NULL,
@@ -617,7 +617,7 @@ compute_K_by_boxes_sparse_nosymm(const BasisInfoStruct& basisInfo,
   // set up CSR K matrix for result
   // note that this call will allocate memory, we must call ergo_CSR_destroy later.
   if(create_CSR_for_K(basisInfo,
-		      &integralInfo,
+		      integralInfo,
 		      J_K_params,
 		      &dens_CSR,
 		      &K_CSR,
@@ -627,7 +627,7 @@ compute_K_by_boxes_sparse_nosymm(const BasisInfoStruct& basisInfo,
       return -1;
     }
   if(create_CSR_for_K(basisInfo,
-		      &integralInfo,
+		      integralInfo,
 		      J_K_params,
 		      &dens_CSR,
 		      &K_CSR_2,
@@ -641,7 +641,7 @@ compute_K_by_boxes_sparse_nosymm(const BasisInfoStruct& basisInfo,
 
   output_current_memory_usage(LOG_AREA_SCF, "Before calling compute_K_by_boxes");
   if(compute_K_by_boxes(basisInfo,
-			&integralInfo,
+			integralInfo,
 			CAM_params,
 			J_K_params,
 			NULL,
