@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -58,6 +58,18 @@ void getMatrixPermutation(const BasisInfoStruct& basisInfo,
 			  int factor3,
 			  std::vector<int> & permutation,
 			  std::vector<int> & inversePermutation);
+void getMatrixPermutationOnlyFactor2(const std::vector<ergo_real> & xcoords,
+				     const std::vector<ergo_real> & ycoords,
+				     const std::vector<ergo_real> & zcoords,
+				     int sparse_block_size_lowest,
+				     int first_factor, // this factor may be different from 2, all other factors are always 2.
+				     std::vector<int> & permutation,
+				     std::vector<int> & inversePermutation);
+void getMatrixPermutationOnlyFactor2(const BasisInfoStruct& basisInfo,
+				     int sparse_block_size_lowest,
+				     int first_factor, // this factor may be different from 2, all other factors are always 2.
+				     std::vector<int> & permutation,
+				     std::vector<int> & inversePermutation);
 
 #endif
 void fill_matrix_with_random_numbers(int n, symmMatrix & M);
@@ -66,6 +78,9 @@ void add_random_diag_perturbation(int n,
 				  symmMatrix & M, 
 				  ergo_real eps);
 
+bool check_if_matrix_contains_strange_elements(const symmMatrix & M,
+                                               std::vector<int> const & inversePermutationHML);
+
 void output_matrix(int n, const ergo_real* matrix, const char* matrixName);
 
 template<class Tmatrix>
