@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,14 +27,8 @@
 
 #include <memory.h>
 
-#ifdef __cplusplus
-#define EXTERN_C extern "C"
-#else
-#define EXTERN_C
-#endif
-
-EXTERN_C void* ergo_malloc(size_t size);
-EXTERN_C void ergo_free(void* p);
-EXTERN_C void report_memory_status(void);
+void* ergo_malloc(size_t size);
+void ergo_free(void* p);
+void report_memory_status(void);
 
 #define ergo_new(cnt,type) ((type*)ergo_malloc((cnt)*sizeof(type)))
