@@ -1,6 +1,6 @@
-/* Ergo, version 3.2, a program for linear scaling electronic structure
+/* Ergo, version 3.3, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2012 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,12 +33,6 @@
 #include <sys/time.h>
 #include <sys/resource.h>
 
-#ifdef __cplusplus
-#define EXTERN_C extern "C"
-#else
-#define EXTERN_C
-#endif
-
 
 #define MAX_HOST_NAME_LEN 100
 
@@ -54,19 +48,20 @@ typedef struct
   char s[MAX_WORKING_DIRECTORY_LEN];
 } working_directory_struct;
 
-EXTERN_C void get_host_name(host_name_struct* result);
+void get_host_name(host_name_struct* result);
 
-EXTERN_C void get_working_directory(working_directory_struct* result);
+void get_working_directory(working_directory_struct* result);
 
-EXTERN_C int get_memory_usage_by_ps(double* virtualMemoryGigaBytes, double* residentMemoryGigaBytes);
+int get_memory_usage_by_ps(double* virtualMemoryGigaBytes, double* residentMemoryGigaBytes);
 
-EXTERN_C int get_memory_usage_by_procfile(double* virtualMemGigaBytes,
+int get_memory_usage_by_procfile(double* virtualMemGigaBytes,
 					  double* residentMemGigaBytes,
 					  double* virtualMemPeakGigaBytes);
 
-EXTERN_C int generate_unique_random_filename(char* result, unsigned n);
+int generate_unique_random_filename(char* result, unsigned n);
+
+long int get_file_size(const char* fileName);
 
-#ifdef __cplusplus
 #include <stdexcept>
 #include "output.h"
 #include "realtype.h"
@@ -115,6 +110,5 @@ namespace Util {
   };
 }
 
-#endif
 
 #endif /* UTILITIES_HEADER */
